@@ -29,9 +29,8 @@
 
 #include <mpd/idle.h>
 
-#include <glib.h>
-
 #include <string.h>
+#include <stdlib.h>
 
 static constexpr char chat_channel[] = "chat";
 
@@ -39,6 +38,8 @@ class ChatPage final : public TextPage {
 	unsigned last_connection_id = 0;
 	bool was_supported = false;
 
+	std::string prefix;
+
 public:
 	ChatPage(ScreenManager &_screen, WINDOW *w, Size size)
 		:TextPage(_screen, w, size) {}
@@ -48,12 +49,17 @@ private:
 
 	void ProcessMessage(const struct mpd_message &message);
 
+	gcc_pure
+	const std::string &GetPrefix() noexcept;
+
+	void SendMessage(struct mpdclient &c, const char *msg) noexcept;
+
 public:
 	/* virtual methods from class Page */
-	void Update(struct mpdclient &c, unsigned events) override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 
-	const char *GetTitle(char *, size_t) const override {
+	const char *GetTitle(char *, size_t) const noexcept override {
 		return _("Chat");
 	}
 };
@@ -69,9 +75,6 @@ ChatPage::CheckChatSupport(struct mpdclient &c)
 
 	last_connection_id = c.connection_id;
 
-	if (c.connection == nullptr)
-		return (was_supported = false);
-
 	/* mpdclient_get_connection? */
 	if (!mpdclient_cmd_subscribe(&c, chat_channel))
 		return (was_supported = false);
@@ -93,13 +96,13 @@ ChatPage::ProcessMessage(const struct mpd_message &message)
 	   client-to-client messages anywhere else */
 	assert(strcmp(mpd_message_get_channel(&message), chat_channel) == 0);
 
-	Append(Utf8ToLocale(mpd_message_get_text(&message)).c_str());
+	Append(mpd_message_get_text(&message));
 
 	SetDirty();
 }
 
 void
-ChatPage::Update(struct mpdclient &c, unsigned events)
+ChatPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (CheckChatSupport(c) && (events & MPD_IDLE_MESSAGE)) {
 		auto *connection = c.GetConnection();
@@ -121,33 +124,32 @@ ChatPage::Update(struct mpdclient &c, unsigned events)
 	}
 }
 
-static char *
-screen_chat_get_prefix()
+const std::string &
+ChatPage::GetPrefix() noexcept
 {
-	static char *prefix = nullptr;
-
-	if (prefix)
+	if (!prefix.empty())
 		return prefix;
 
 	if (!options.chat_prefix.empty()) {
 		/* Options are encoded in the "locale" charset */
-		prefix = locale_to_utf8(options.chat_prefix.c_str());
+		prefix = LocaleToUtf8(options.chat_prefix.c_str()).c_str();
 		return prefix;
 	}
 
-	prefix = g_strconcat("<", g_get_user_name(), "> ", nullptr);
+	const char *user = getenv("USER");
+	if (user == nullptr)
+		user = "nobody";
+
+	prefix = std::string("<") + user + "> ";
 	return prefix;
 }
 
-static void
-screen_chat_send_message(struct mpdclient *c, const char *msg)
+void
+ChatPage::SendMessage(struct mpdclient &c, const char *msg) noexcept
 {
-	char *prefix = screen_chat_get_prefix();
-	char *full_msg = g_strconcat(prefix, LocaleToUtf8(msg).c_str(),
-				     nullptr);
+	const std::string full_msg = GetPrefix() + LocaleToUtf8(msg).c_str();
 
-	(void) mpdclient_cmd_send_message(c, chat_channel, full_msg);
-	g_free(full_msg);
+	(void) mpdclient_cmd_send_message(&c, chat_channel, full_msg.c_str());
 }
 
 bool
@@ -164,7 +166,7 @@ ChatPage::OnCommand(struct mpdclient &c, Command cmd)
 			return true;
 
 		if (CheckChatSupport(c))
-			screen_chat_send_message(&c, message.c_str());
+			SendMessage(c, message.c_str());
 		else
 			screen_status_message(_("Message could not be sent"));
 
