@@ -39,8 +39,6 @@
 
 #include <mpd/client.h>
 
-#include <glib.h>
-
 #include <string.h>
 
 #define BUFSIZE 1024
@@ -49,7 +47,7 @@
 #define HIGHLIGHT  (0x01)
 #endif
 
-FileListPage::~FileListPage()
+FileListPage::~FileListPage() noexcept
 {
 	delete filelist;
 }
@@ -79,7 +77,7 @@ screen_browser_sync_highlights(FileList *fl, const MpdQueue *playlist)
 
 const char *
 FileListPage::GetListItemText(char *buffer, size_t size,
-			      unsigned idx) const
+			      unsigned idx) const noexcept
 {
 	assert(filelist != nullptr);
 	assert(idx < filelist->size());
@@ -93,8 +91,7 @@ FileListPage::GetListItemText(char *buffer, size_t size,
 	if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_DIRECTORY) {
 		const auto *dir = mpd_entity_get_directory(entity);
 		const char *name = GetUriFilename(mpd_directory_get_path(dir));
-		g_strlcpy(buffer, Utf8ToLocale(name).c_str(), size);
-		return buffer;
+		return utf8_to_locale(name, buffer, size);
 	} else if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
 		const auto *song = mpd_entity_get_song(entity);
 
@@ -103,8 +100,7 @@ FileListPage::GetListItemText(char *buffer, size_t size,
 	} else if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_PLAYLIST) {
 		const auto *playlist = mpd_entity_get_playlist(entity);
 		const char *name = GetUriFilename(mpd_playlist_get_path(playlist));
-		g_strlcpy(buffer, Utf8ToLocale(name).c_str(), size);
-		return buffer;
+		return utf8_to_locale(name, buffer, size);
 	}
 
 	return "Error: Unknown entry!";
@@ -266,7 +262,7 @@ browser_select_entry(struct mpdclient *c, FileListEntry *entry,
 		entry->flags |= HIGHLIGHT;
 #endif
 
-		if (mpdclient_cmd_add(c, song)) {
+		if (c->RunAdd(*song)) {
 			char buf[BUFSIZE];
 
 			strfsong(buf, BUFSIZE,
@@ -282,7 +278,7 @@ browser_select_entry(struct mpdclient *c, FileListEntry *entry,
 		entry->flags &= ~HIGHLIGHT;
 
 		while ((idx = c->playlist.FindByUri(mpd_song_get_uri(song))) >= 0)
-			mpdclient_cmd_delete(c, idx);
+			c->RunDelete(idx);
 #endif
 	}
 
@@ -382,11 +378,11 @@ FileListPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SetDirty();
 		return true;
 	case Command::LIST_JUMP:
-		screen_jump(screen, &lw, *this, *this);
+		screen_jump(screen, lw, *this, *this);
 		SetDirty();
 		return true;
 
@@ -482,7 +478,7 @@ screen_browser_paint_playlist(WINDOW *w, unsigned width,
 void
 FileListPage::PaintListItem(WINDOW *w, unsigned i,
 			    unsigned y, unsigned width,
-			    bool selected) const
+			    bool selected) const noexcept
 {
 	assert(filelist != nullptr);
 	assert(i < filelist->size());
@@ -533,7 +529,7 @@ FileListPage::PaintListItem(WINDOW *w, unsigned i,
 }
 
 void
-FileListPage::Paint() const
+FileListPage::Paint() const noexcept
 {
 	lw.Paint(*this);
 }
