@@ -22,7 +22,6 @@
 #include "screen_status.hxx"
 #include "FileBrowserPage.hxx"
 #include "SongPage.hxx"
-#include "Event.hxx"
 #include "i18n.h"
 #include "Options.hxx"
 #include "mpdclient.hxx"
@@ -32,6 +31,8 @@
 #include "screen_utils.hxx"
 #include "ncu.hxx"
 
+#include <boost/asio/steady_timer.hpp>
+
 #include <string>
 
 #include <assert.h>
@@ -40,7 +41,6 @@
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glib.h>
 #include <unistd.h>
 #include <stdio.h>
 
@@ -63,16 +63,21 @@ class LyricsPage final : public TextPage {
 
 	PluginCycle *loader = nullptr;
 
-	guint loader_timeout = 0;
+	boost::asio::steady_timer loader_timeout;
 
 public:
 	LyricsPage(ScreenManager &_screen, WINDOW *w, Size size)
-		:TextPage(_screen, w, size) {}
+		:TextPage(_screen, w, size),
+		 loader_timeout(_screen.get_io_service()) {}
 
 	~LyricsPage() override {
 		Cancel();
 	}
 
+	auto &get_io_service() noexcept {
+		return loader_timeout.get_io_service();
+	}
+
 private:
 	void Cancel();
 
@@ -88,7 +93,14 @@ private:
 
 	void Set(const char *s);
 
-	void Load(const struct mpd_song *song);
+	void Load(const struct mpd_song &song) noexcept;
+	void MaybeLoad(const struct mpd_song &new_song) noexcept;
+
+	void MaybeLoad(const struct mpd_song *new_song) noexcept {
+		if (new_song != nullptr)
+			MaybeLoad(*new_song);
+	}
+
 	void Reload();
 
 	bool Save();
@@ -106,16 +118,14 @@ private:
 	void PluginCallback(std::string &&result, bool success,
 			    const char *plugin_name);
 
-	bool TimeoutCallback();
+	void OnTimeout(const boost::system::error_code &error) noexcept;
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-
-	void Update(struct mpdclient &c, unsigned events) override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-
-	const char *GetTitle(char *, size_t) const override;
+	const char *GetTitle(char *, size_t) const noexcept override;
 };
 
 void
@@ -126,10 +136,7 @@ LyricsPage::Cancel()
 		loader = nullptr;
 	}
 
-	if (loader_timeout != 0) {
-		g_source_remove(loader_timeout);
-		loader_timeout = 0;
-	}
+	loader_timeout.cancel();
 
 	plugin_name.clear();
 
@@ -225,7 +232,10 @@ LyricsPage::PluginCallback(std::string &&result, const bool success,
 {
 	assert(loader != nullptr);
 
-	plugin_name = _plugin_name;
+	if (_plugin_name != nullptr)
+		plugin_name = _plugin_name;
+	else
+		plugin_name.clear();
 
 	/* Display result, which may be lyrics or error messages */
 	Set(result.c_str());
@@ -239,55 +249,63 @@ LyricsPage::PluginCallback(std::string &&result, const bool success,
 		screen_status_message (_("No lyrics"));
 	}
 
-	if (loader_timeout != 0) {
-		g_source_remove(loader_timeout);
-		loader_timeout = 0;
-	}
+	loader_timeout.cancel();
 
 	plugin_stop(loader);
 	loader = nullptr;
 }
 
-inline bool
-LyricsPage::TimeoutCallback()
+void
+LyricsPage::OnTimeout(const boost::system::error_code &error) noexcept
 {
+	if (error)
+		return;
+
 	plugin_stop(loader);
 	loader = nullptr;
 
 	screen_status_printf(_("Lyrics timeout occurred after %d seconds"),
-			     options.lyrics_timeout);
-
-	loader_timeout = 0;
-	return false;
+			     (int)std::chrono::duration_cast<std::chrono::seconds>(options.lyrics_timeout).count());
 }
 
 void
-LyricsPage::Load(const struct mpd_song *_song)
+LyricsPage::Load(const struct mpd_song &_song) noexcept
 {
-	assert(_song != nullptr);
-
 	Cancel();
 	Clear();
 
-	song = mpd_song_dup(_song);
+	song = mpd_song_dup(&_song);
 	artist = mpd_song_get_tag(song, MPD_TAG_ARTIST, 0);
 	title = mpd_song_get_tag(song, MPD_TAG_TITLE, 0);
 
-	loader = lyrics_load(artist, title, PluginCallback, this);
+	loader = lyrics_load(get_io_service(),
+			     artist, title, PluginCallback, this);
 
-	if (options.lyrics_timeout != 0) {
-		loader_timeout = ScheduleTimeout<LyricsPage,
-						 &LyricsPage::TimeoutCallback>(std::chrono::seconds(options.lyrics_timeout),
-									       *this);
+	if (options.lyrics_timeout > std::chrono::steady_clock::duration::zero()) {
+		boost::system::error_code error;
+		loader_timeout.expires_from_now(options.lyrics_timeout,
+						error);
+		loader_timeout.async_wait(std::bind(&LyricsPage::OnTimeout, this,
+						     std::placeholders::_1));
 	}
 }
 
+void
+LyricsPage::MaybeLoad(const struct mpd_song &new_song) noexcept
+{
+	if (song == nullptr ||
+	    strcmp(mpd_song_get_uri(&new_song),
+		   mpd_song_get_uri(song)) != 0)
+		Load(new_song);
+}
+
 void
 LyricsPage::Reload()
 {
 	if (loader == nullptr && artist != nullptr && title != nullptr) {
 		reloading = true;
-		loader = lyrics_load(artist, title, PluginCallback, nullptr);
+		loader = lyrics_load(get_io_service(),
+				     artist, title, PluginCallback, nullptr);
 		Repaint();
 	}
 }
@@ -299,16 +317,12 @@ lyrics_screen_init(ScreenManager &_screen, WINDOW *w, Size size)
 }
 
 void
-LyricsPage::OnOpen(struct mpdclient &c)
+LyricsPage::OnOpen(struct mpdclient &c) noexcept
 {
 	const struct mpd_song *next_song_c =
-		next_song != nullptr ? next_song : c.song;
+		next_song != nullptr ? next_song : c.GetPlayingSong();
 
-	if (next_song_c != nullptr &&
-	    (song == nullptr ||
-	     strcmp(mpd_song_get_uri(next_song_c),
-		    mpd_song_get_uri(song)) != 0))
-		Load(next_song_c);
+	MaybeLoad(next_song_c);
 
 	if (next_song != nullptr) {
 		mpd_song_free(next_song);
@@ -317,20 +331,14 @@ LyricsPage::OnOpen(struct mpdclient &c)
 }
 
 void
-LyricsPage::Update(struct mpdclient &c, unsigned)
+LyricsPage::Update(struct mpdclient &c, unsigned) noexcept
 {
-	if (!follow)
-		return;
-
-	if (c.song != nullptr &&
-	    (song == nullptr ||
-	     strcmp(mpd_song_get_uri(c.song),
-		    mpd_song_get_uri(song)) != 0))
-		Load(c.song);
+	if (follow)
+		MaybeLoad(c.GetPlayingSong());
 }
 
 const char *
-LyricsPage::GetTitle(char *str, size_t size) const
+LyricsPage::GetTitle(char *str, size_t size) const noexcept
 {
 	if (loader != nullptr) {
 		snprintf(str, size, "%s (%s)",
@@ -384,7 +392,7 @@ LyricsPage::Edit()
 	int status;
 	pid_t pid = fork();
 	if (pid == -1) {
-		screen_status_printf(("%s (%s)"), _("Can't start editor"), g_strerror(errno));
+		screen_status_printf(("%s (%s)"), _("Can't start editor"), strerror(errno));
 		ncu_init();
 		return;
 	} else if (pid == 0) {
@@ -448,8 +456,8 @@ LyricsPage::OnCommand(struct mpdclient &c, Command cmd)
 		}
 		return true;
 	case Command::LYRICS_UPDATE:
-		if (c.song != nullptr)
-			Load(c.song);
+		if (c.GetPlayingSong() != nullptr)
+			Load(*c.GetPlayingSong());
 		return true;
 	case Command::EDIT:
 		Edit();
