@@ -18,6 +18,7 @@
  */
 
 #include "config.h"
+#include "Instance.hxx"
 #include "ncmpc.hxx"
 #include "mpdclient.hxx"
 #include "callbacks.hxx"
@@ -33,9 +34,8 @@
 #include "strfsong.hxx"
 #include "i18n.h"
 #include "player_command.hxx"
-#include "keyboard.hxx"
-#include "lirc.hxx"
-#include "signals.hxx"
+#include "util/ScopeExit.hxx"
+#include "util/StringUTF8.hxx"
 
 #ifndef NCMPC_MINI
 #include "conf.hxx"
@@ -47,8 +47,6 @@
 
 #include <mpd/client.h>
 
-#include <glib.h>
-
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -59,18 +57,10 @@
 #include <locale.h>
 #endif
 
-/* time between mpd updates [ms] */
-static const guint update_interval = 500;
-
 #define BUFSIZE 1024
 
+static Instance *global_instance;
 static struct mpdclient *mpd = nullptr;
-static GMainLoop *main_loop;
-static guint reconnect_source_id, update_source_id;
-
-#ifndef NCMPC_MINI
-static guint check_key_bindings_source_id;
-#endif
 
 ScreenManager *screen;
 
@@ -78,11 +68,11 @@ ScreenManager *screen;
 static void
 update_xterm_title()
 {
-	const struct mpd_song *song = mpd->song;
+	const struct mpd_song *song = mpd->GetPlayingSong();
 
 	char tmp[BUFSIZE];
 	const char *new_title = nullptr;
-	if (!options.xterm_title_format.empty() && mpd->playing_or_paused && song)
+	if (!options.xterm_title_format.empty() && song != nullptr)
 		new_title = strfsong(tmp, BUFSIZE,
 				     options.xterm_title_format.c_str(), song) > 0
 			? tmp
@@ -93,35 +83,12 @@ update_xterm_title()
 
 	static char title[BUFSIZE];
 	if (strncmp(title, new_title, BUFSIZE)) {
-		g_strlcpy(title, new_title, BUFSIZE);
+		strcpy(title, new_title);
 		set_xterm_title(title);
 	}
 }
 #endif
 
-static gboolean
-timer_mpd_update(gpointer data);
-
-static void
-enable_update_timer()
-{
-	if (update_source_id != 0)
-		return;
-
-	update_source_id = g_timeout_add(update_interval,
-					 timer_mpd_update, nullptr);
-}
-
-static void
-disable_update_timer()
-{
-	if (update_source_id == 0)
-		return;
-
-	g_source_remove(update_source_id);
-	update_source_id = 0;
-}
-
 static bool
 should_enable_update_timer()
 {
@@ -132,52 +99,45 @@ static void
 auto_update_timer()
 {
 	if (should_enable_update_timer())
-		enable_update_timer();
+		global_instance->EnableUpdateTimer();
 	else
-		disable_update_timer();
+		global_instance->DisableUpdateTimer();
 }
 
-static void
-do_mpd_update()
+void
+Instance::UpdateClient() noexcept
 {
-	if (mpd->IsConnected() &&
-	    (mpd->events != 0 || mpd->playing))
-		mpd->Update();
+	if (client.IsConnected() &&
+	    (client.events != 0 || client.playing))
+		client.Update();
 
 #ifndef NCMPC_MINI
 	if (options.enable_xterm_title)
 		update_xterm_title();
 #endif
 
-	screen->Update(*mpd);
-	mpd->events = (enum mpd_idle)0;
+	screen_manager.Update(client, seek);
+	client.events = (enum mpd_idle)0;
 }
 
-/**
- * This timer is installed when the connection to the MPD server is
- * broken.  It tries to recover by reconnecting periodically.
- */
-static gboolean
-timer_reconnect(gcc_unused gpointer data)
+void
+Instance::OnReconnectTimer(const boost::system::error_code &error) noexcept
 {
-	assert(mpd->IsDead());
+	if (error)
+		return;
 
-	reconnect_source_id = 0;
+	assert(client.IsDead());
 
 	screen_status_printf(_("Connecting to %s"),
-			     mpd->GetSettingsName().c_str());
+			     client.GetSettingsName().c_str());
 	doupdate();
 
-	mpd->Connect();
-
-	return false;
+	client.Connect();
 }
 
 void
 mpdclient_connected_callback()
 {
-	assert(reconnect_source_id == 0);
-
 #ifndef NCMPC_MINI
 	/* quit if mpd is pre 0.14 - song id not supported by mpd */
 	auto *connection = mpd->GetConnection();
@@ -191,8 +151,7 @@ mpdclient_connected_callback()
 		doupdate();
 
 		/* try again after 30 seconds */
-		reconnect_source_id =
-			g_timeout_add_seconds(30, timer_reconnect, nullptr);
+		global_instance->ScheduleReconnect(std::chrono::seconds(30));
 		return;
 	}
 #endif
@@ -200,7 +159,7 @@ mpdclient_connected_callback()
 	screen->status_bar.ClearMessage();
 	doupdate();
 
-	do_mpd_update();
+	global_instance->UpdateClient();
 
 	auto_update_timer();
 }
@@ -208,20 +167,16 @@ mpdclient_connected_callback()
 void
 mpdclient_failed_callback()
 {
-	assert(reconnect_source_id == 0);
-
 	/* try again in 5 seconds */
-	reconnect_source_id = g_timeout_add_seconds(5, timer_reconnect, nullptr);
+	global_instance->ScheduleReconnect(std::chrono::seconds(5));
 }
 
 void
 mpdclient_lost_callback()
 {
-	assert(reconnect_source_id == 0);
+	screen->Update(*mpd, global_instance->GetSeek());
 
-	screen->Update(*mpd);
-
-	reconnect_source_id = g_timeout_add_seconds(1, timer_reconnect, nullptr);
+	global_instance->ScheduleReconnect(std::chrono::seconds(1));
 }
 
 /**
@@ -236,21 +191,23 @@ mpdclient_idle_callback(gcc_unused unsigned events)
 		update_xterm_title();
 #endif
 
-	screen->Update(*mpd);
+	screen->Update(*mpd, global_instance->GetSeek());
 	auto_update_timer();
 }
 
-static gboolean
-timer_mpd_update(gcc_unused gpointer data)
+void
+Instance::OnUpdateTimer(const boost::system::error_code &error) noexcept
 {
-	do_mpd_update();
+	if (error)
+		return;
+
+	assert(pending_update_timer);
+	pending_update_timer = false;
+
+	UpdateClient();
 
 	if (should_enable_update_timer())
-		return true;
-	else {
-		update_source_id = 0;
-		return false;
-	}
+		ScheduleUpdateTimer();
 }
 
 void begin_input_event()
@@ -259,25 +216,25 @@ void begin_input_event()
 
 void end_input_event()
 {
-	screen->Update(*mpd);
+	screen->Update(*mpd, global_instance->GetSeek());
 	mpd->events = (enum mpd_idle)0;
 
 	auto_update_timer();
 }
 
 bool
-do_input_event(Command cmd)
+do_input_event(boost::asio::io_service &io_service, Command cmd)
 {
 	if (cmd == Command::QUIT) {
-		g_main_loop_quit(main_loop);
+		io_service.stop();
 		return false;
 	}
 
-	screen->OnCommand(*mpd, cmd);
+	screen->OnCommand(*mpd, global_instance->GetSeek(), cmd);
 
 	if (cmd == Command::VOLUME_UP || cmd == Command::VOLUME_DOWN)
 		/* make sure we don't update the volume yet */
-		disable_update_timer();
+		global_instance->DisableUpdateTimer();
 
 	return true;
 }
@@ -287,7 +244,7 @@ do_input_event(Command cmd)
 void
 do_mouse_event(Point p, mmask_t bstate)
 {
-	screen->OnMouse(*mpd, p, bstate);
+	screen->OnMouse(*mpd, global_instance->GetSeek(), p, bstate);
 }
 
 #endif
@@ -297,22 +254,24 @@ do_mouse_event(Point p, mmask_t bstate)
  * Check the configured key bindings for errors, and display a status
  * message every 10 seconds.
  */
-static gboolean
-timer_check_key_bindings(gcc_unused gpointer data)
+void
+Instance::OnCheckKeyBindings(const boost::system::error_code &error) noexcept
 {
+	if (error)
+		return;
+
 	char buf[256];
 
-	if (GetGlobalKeyBindings().Check(buf, sizeof(buf))) {
+	if (GetGlobalKeyBindings().Check(buf, sizeof(buf)))
 		/* no error: disable this timer for the rest of this
 		   process */
-		check_key_bindings_source_id = 0;
-		return false;
-	}
+		return;
 
 	screen_status_message(buf);
 
 	doupdate();
-	return true;
+
+	ScheduleCheckKeyBindings();
 }
 #endif
 
@@ -320,18 +279,18 @@ int
 main(int argc, const char *argv[])
 {
 #ifdef ENABLE_LOCALE
-#ifndef ENABLE_NLS
-	gcc_unused
-#endif
-	const char *charset = nullptr;
 	/* time and date formatting */
 	setlocale(LC_TIME,"");
 	/* care about sorting order etc */
 	setlocale(LC_COLLATE,"");
 	/* charset */
 	setlocale(LC_CTYPE,"");
+#ifdef HAVE_ICONV
 	/* initialize charset conversions */
-	charset = charset_init();
+	charset_init();
+#endif
+
+	const ScopeInitUTF8 init_utf8;
 
 	/* initialize i18n support */
 #endif
@@ -339,9 +298,6 @@ main(int argc, const char *argv[])
 #ifdef ENABLE_NLS
 	setlocale(LC_MESSAGES, "");
 	bindtextdomain(GETTEXT_PACKAGE, LOCALE_DIR);
-#ifdef ENABLE_LOCALE
-	bind_textdomain_codeset(GETTEXT_PACKAGE, charset);
-#endif
 	textdomain(GETTEXT_PACKAGE);
 #endif
 
@@ -365,66 +321,30 @@ main(int argc, const char *argv[])
 	lyrics_init();
 #endif
 
-	/* create mpdclient instance */
-	struct mpdclient client(options.host.empty() ? nullptr : options.host.c_str(),
-				options.port,
-				options.timeout_ms,
-				options.password.empty() ? nullptr : options.password.c_str());
-	mpd = &client;
+	/* create the global Instance */
+	Instance instance;
+	global_instance = &instance;
+	mpd = &instance.GetClient();
+	screen = &instance.GetScreenManager();
 
-	/* initialize curses */
-	ScreenManager screen_manager;
-	screen_manager.Init(&client);
-	screen = &screen_manager;
-
-	/* the main loop */
-	main_loop = g_main_loop_new(nullptr, false);
-
-	/* watch out for keyboard input */
-	keyboard_init(screen_manager.main_window.w);
-
-	/* watch out for lirc input */
-	ncmpc_lirc_init();
-
-	signals_init(main_loop, screen_manager);
+	AtScopeExit() {
+		/* this must be executed after ~Instance(), so we're
+		   using AtScopeExit() to do the trick */
+#ifndef NCMPC_MINI
+		set_xterm_title("");
+#endif
+		printf("\n");
+	};
 
 	/* attempt to connect */
-	reconnect_source_id = g_idle_add(timer_reconnect, nullptr);
+	instance.ScheduleReconnect(std::chrono::seconds(0));
 
 	auto_update_timer();
 
 #ifndef NCMPC_MINI
-	check_key_bindings_source_id =
-		g_timeout_add_seconds(10, timer_check_key_bindings, nullptr);
+	instance.ScheduleCheckKeyBindings();
 #endif
 
-	screen_manager.Update(client);
-
-	g_main_loop_run(main_loop);
-	g_main_loop_unref(main_loop);
-
-	/* cleanup */
-
-	cancel_seek_timer();
-
-	disable_update_timer();
-
-	if (reconnect_source_id != 0)
-		g_source_remove(reconnect_source_id);
-
-#ifndef NCMPC_MINI
-	if (check_key_bindings_source_id != 0)
-		g_source_remove(check_key_bindings_source_id);
-#endif
-
-	signals_deinit();
-	ncmpc_lirc_deinit();
-
-	screen_manager.Exit();
-#ifndef NCMPC_MINI
-	set_xterm_title("");
-#endif
-	printf("\n");
-
+	instance.Run();
 	return 0;
 }
