@@ -54,14 +54,14 @@ private:
 
 public:
 	/* virtual methods from class Page */
-	void Paint() const override;
-	void Update(struct mpdclient &c, unsigned events) override;
+	void Paint() const noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 
 	/* virtual methods from class ListRenderer */
 	void PaintListItem(WINDOW *w, unsigned i, unsigned y, unsigned width,
-			   bool selected) const override;
+			   bool selected) const noexcept override;
 };
 
 bool
@@ -140,7 +140,7 @@ outputs_init(ScreenManager &, WINDOW *w, Size size)
 }
 
 const char *
-OutputsPage::GetTitle(gcc_unused char *str, gcc_unused size_t size) const
+OutputsPage::GetTitle(char *, size_t) const noexcept
 {
 	return _("Outputs");
 }
@@ -148,7 +148,7 @@ OutputsPage::GetTitle(gcc_unused char *str, gcc_unused size_t size) const
 void
 OutputsPage::PaintListItem(WINDOW *w, unsigned i,
 			   gcc_unused unsigned y, unsigned width,
-			   bool selected) const
+			   bool selected) const noexcept
 {
 	assert(i < items.size());
 	const auto *output = items[i].get();
@@ -160,13 +160,13 @@ OutputsPage::PaintListItem(WINDOW *w, unsigned i,
 }
 
 void
-OutputsPage::Paint() const
+OutputsPage::Paint() const noexcept
 {
 	lw.Paint(*this);
 }
 
 void
-OutputsPage::Update(struct mpdclient &c, unsigned events)
+OutputsPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (events & MPD_IDLE_OUTPUT) {
 		Clear();
