@@ -18,7 +18,8 @@
  */
 
 #include "ProgressBar.hxx"
-#include "colors.hxx"
+#include "Styles.hxx"
+#include "Options.hxx"
 
 #include <assert.h>
 
@@ -26,7 +27,10 @@ ProgressBar::ProgressBar(Point p, unsigned _width)
 	:window(p, {_width, 1u})
 {
 	leaveok(window.w, true);
-	wbkgd(window.w, COLOR_PAIR(COLOR_PROGRESSBAR));
+#ifdef ENABLE_COLORS
+	if (options.enable_colors)
+		wbkgd(window.w, COLOR_PAIR(Style::PROGRESSBAR));
+#endif
 }
 
 void
@@ -35,7 +39,7 @@ ProgressBar::Paint() const
 	if (max > 0) {
 		assert(width < window.size.width);
 
-		colors_use(window.w, COLOR_PROGRESSBAR);
+		SelectStyle(window.w, Style::PROGRESSBAR);
 
 		if (width > 0)
 			mvwhline(window.w, 0, 0, '=', width);
@@ -44,12 +48,12 @@ ProgressBar::Paint() const
 		unsigned x = width + 1;
 
 		if (x < window.size.width) {
-			colors_use(window.w, COLOR_PROGRESSBAR_BACKGROUND);
+			SelectStyle(window.w, Style::PROGRESSBAR_BACKGROUND);
 			mvwhline(window.w, 0, x, ACS_HLINE, window.size.width - x);
 		}
 	} else {
 		/* no progress bar, just a simple horizontal line */
-		colors_use(window.w, COLOR_LINE);
+		SelectStyle(window.w, Style::LINE);
 		mvwhline(window.w, 0, 0, ACS_HLINE, window.size.width);
 	}
 
