@@ -27,7 +27,6 @@
 #include "screen_find.hxx"
 #include "save_playlist.hxx"
 #include "config.h"
-#include "Event.hxx"
 #include "i18n.h"
 #include "charset.hxx"
 #include "Options.hxx"
@@ -49,6 +48,8 @@
 
 #include <mpd/client.h>
 
+#include <boost/asio/steady_timer.hpp>
+
 #include <set>
 #include <string>
 
@@ -63,10 +64,11 @@ class QueuePage final : public ListPage, ListRenderer, ListText {
 	mutable class hscroll hscroll;
 #endif
 
+	boost::asio::steady_timer hide_cursor_timer;
+
 	MpdQueue *playlist = nullptr;
 	int current_song_id = -1;
 	int selected_song_id = -1;
-	guint timer_hide_cursor_id = 0;
 
 	unsigned last_connection_id = 0;
 	std::string connection_name;
@@ -77,10 +79,12 @@ public:
 	QueuePage(ScreenManager &_screen, WINDOW *w,
 		  Size size)
 		:ListPage(w, size),
-		 screen(_screen)
+		 screen(_screen),
 #ifndef NCMPC_MINI
-		, hscroll(w, options.scroll_sep.c_str())
+		 hscroll(screen.get_io_service(),
+			 w, options.scroll_sep.c_str()),
 #endif
+		 hide_cursor_timer(screen.get_io_service())
 	{
 	}
 
@@ -101,39 +105,40 @@ private:
 
 	bool OnSongChange(const struct mpd_status *status);
 
-	bool OnHideCursorTimer();
+	void OnHideCursorTimer(const boost::system::error_code &error) noexcept;
 
 	void ScheduleHideCursor() {
-		assert(options.hide_cursor > 0);
-		assert(timer_hide_cursor_id == 0);
+		assert(options.hide_cursor > std::chrono::steady_clock::duration::zero());
 
-		timer_hide_cursor_id = ScheduleTimeout<QueuePage,
-						       &QueuePage::OnHideCursorTimer>(std::chrono::seconds(options.hide_cursor),
-										      *this);
+		boost::system::error_code error;
+		hide_cursor_timer.expires_from_now(options.hide_cursor,
+						   error);
+		hide_cursor_timer.async_wait(std::bind(&QueuePage::OnHideCursorTimer, this,
+						       std::placeholders::_1));
 	}
 
 	/* virtual methods from class ListRenderer */
 	void PaintListItem(WINDOW *w, unsigned i,
 			   unsigned y, unsigned width,
-			   bool selected) const override;
+			   bool selected) const noexcept override;
 
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const override;
+				    unsigned i) const noexcept override;
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-	void OnClose() override;
-	void Paint() const override;
-	void Update(struct mpdclient &c, unsigned events) override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void OnClose() noexcept override;
+	void Paint() const noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 
 #ifdef HAVE_GETMOUSE
 	bool OnMouse(struct mpdclient &c, Point p, mmask_t bstate) override;
 #endif
 
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 };
 
 const struct mpd_song *
@@ -177,7 +182,7 @@ QueuePage::RestoreSelection()
 
 const char *
 QueuePage::GetListItemText(char *buffer, size_t size,
-			   unsigned idx) const
+			   unsigned idx) const noexcept
 {
 	assert(idx < playlist->size());
 
@@ -323,13 +328,13 @@ screen_queue_init(ScreenManager &_screen, WINDOW *w, Size size)
 	return std::make_unique<QueuePage>(_screen, w, size);
 }
 
-bool
-QueuePage::OnHideCursorTimer()
+void
+QueuePage::OnHideCursorTimer(const boost::system::error_code &error) noexcept
 {
-	assert(options.hide_cursor > 0);
-	assert(timer_hide_cursor_id != 0);
+	if (error)
+		return;
 
-	timer_hide_cursor_id = 0;
+	assert(options.hide_cursor > std::chrono::steady_clock::duration::zero());
 
 	/* hide the cursor when mpd is playing and the user is inactive */
 
@@ -338,17 +343,14 @@ QueuePage::OnHideCursorTimer()
 		Repaint();
 	} else
 		ScheduleHideCursor();
-
-	return false;
 }
 
 void
-QueuePage::OnOpen(struct mpdclient &c)
+QueuePage::OnOpen(struct mpdclient &c) noexcept
 {
 	playlist = &c.playlist;
 
-	assert(timer_hide_cursor_id == 0);
-	if (options.hide_cursor > 0) {
+	if (options.hide_cursor > std::chrono::steady_clock::duration::zero()) {
 		lw.hide_cursor = false;
 		ScheduleHideCursor();
 	}
@@ -358,12 +360,9 @@ QueuePage::OnOpen(struct mpdclient &c)
 }
 
 void
-QueuePage::OnClose()
+QueuePage::OnClose() noexcept
 {
-	if (timer_hide_cursor_id != 0) {
-		g_source_remove(timer_hide_cursor_id);
-		timer_hide_cursor_id = 0;
-	}
+	hide_cursor_timer.cancel();
 
 #ifndef NCMPC_MINI
 	if (options.scroll)
@@ -372,7 +371,7 @@ QueuePage::OnClose()
 }
 
 const char *
-QueuePage::GetTitle(char *str, size_t size) const
+QueuePage::GetTitle(char *str, size_t size) const noexcept
 {
 	if (connection_name.empty())
 		return _("Queue");
@@ -383,7 +382,7 @@ QueuePage::GetTitle(char *str, size_t size) const
 
 void
 QueuePage::PaintListItem(WINDOW *w, unsigned i, unsigned y, unsigned width,
-			 bool selected) const
+			 bool selected) const noexcept
 {
 	assert(playlist != nullptr);
 	assert(i < playlist->size());
@@ -401,7 +400,7 @@ QueuePage::PaintListItem(WINDOW *w, unsigned i, unsigned y, unsigned width,
 }
 
 void
-QueuePage::Paint() const
+QueuePage::Paint() const noexcept
 {
 #ifndef NCMPC_MINI
 	if (options.scroll)
@@ -412,7 +411,7 @@ QueuePage::Paint() const
 }
 
 void
-QueuePage::Update(struct mpdclient &c, unsigned events)
+QueuePage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	playing = c.playing;
 
@@ -444,7 +443,11 @@ QueuePage::OnMouse(struct mpdclient &c, Point p, mmask_t bstate)
 
 	if (bstate & BUTTON1_DOUBLE_CLICKED) {
 		/* stop */
-		screen.OnCommand(c, Command::STOP);
+
+		auto *connection = c.GetConnection();
+		if (connection != nullptr &&
+		    !mpd_run_stop(connection))
+			c.HandleError();
 		return true;
 	}
 
@@ -464,7 +467,7 @@ QueuePage::OnMouse(struct mpdclient &c, Point p, mmask_t bstate)
 	} else if (bstate & BUTTON3_CLICKED) {
 		/* delete */
 		if (lw.selected == old_selected)
-			mpdclient_cmd_delete(&c, lw.selected);
+			c.RunDelete(lw.selected);
 
 		lw.SetLength(playlist->size());
 	}
@@ -487,12 +490,7 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 
 	lw.hide_cursor = false;
 
-	if (options.hide_cursor > 0) {
-		if (timer_hide_cursor_id != 0) {
-			g_source_remove(timer_hide_cursor_id);
-			timer_hide_cursor_id = 0;
-		}
-
+	if (options.hide_cursor > std::chrono::steady_clock::duration::zero()) {
 		ScheduleHideCursor();
 	}
 
@@ -502,12 +500,18 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 	}
 
 	switch(cmd) {
+		int pos;
+
 	case Command::SCREEN_UPDATE:
 		CenterPlayingItem(c.status, prev_cmd == Command::SCREEN_UPDATE);
 		SetDirty();
 		return false;
 	case Command::SELECT_PLAYING:
-		lw.SetCursor(c.playlist.FindByReference(*c.song));
+		pos = c.GetCurrentSongPos();
+		if (pos < 0)
+			return false;
+
+		lw.SetCursor(pos);
 		SaveSelection();
 		SetDirty();
 		return true;
@@ -516,12 +520,12 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SaveSelection();
 		SetDirty();
 		return true;
 	case Command::LIST_JUMP:
-		screen_jump(screen, &lw, *this, *this);
+		screen_jump(screen, lw, *this, *this);
 		SaveSelection();
 		SetDirty();
 		return true;
@@ -538,13 +542,11 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 
 #ifdef ENABLE_LYRICS_SCREEN
 	case Command::SCREEN_LYRICS:
-		if (lw.selected < c.playlist.size()) {
-			struct mpd_song &selected = c.playlist[lw.selected];
+		if (lw.selected < playlist->size()) {
+			struct mpd_song &selected = (*playlist)[lw.selected];
 			bool follow = false;
 
-			if (c.song &&
-			    !strcmp(mpd_song_get_uri(&selected),
-				    mpd_song_get_uri(c.song)))
+			if (&selected == c.GetPlayingSong())
 				follow = true;
 
 			screen_lyrics_switch(screen, c, selected, follow);
@@ -554,8 +556,8 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 		break;
 #endif
 	case Command::SCREEN_SWAP:
-		if (!c.playlist.empty())
-			screen.Swap(c, &c.playlist[lw.selected]);
+		if (!playlist->empty())
+			screen.Swap(c, &(*playlist)[lw.selected]);
 		else
 			screen.Swap(c, nullptr);
 		return true;
@@ -585,8 +587,7 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 
 	case Command::DELETE:
 		range = lw.GetRange();
-		mpdclient_cmd_delete_range(&c, range.start_index,
-					   range.end_index);
+		c.RunDeleteRange(range.start_index, range.end_index);
 
 		lw.SetCursor(range.start_index);
 		return true;
@@ -621,8 +622,7 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 		if (range.start_index == 0 || range.empty())
 			return false;
 
-		if (!mpdclient_cmd_move(&c, range.end_index - 1,
-					range.start_index - 1))
+		if (!c.RunMove(range.end_index - 1, range.start_index - 1))
 			return true;
 
 		lw.selected--;
@@ -637,11 +637,10 @@ QueuePage::OnCommand(struct mpdclient &c, Command cmd)
 
 	case Command::LIST_MOVE_DOWN:
 		range = lw.GetRange();
-		if (range.end_index >= c.playlist.size())
+		if (range.end_index >= playlist->size())
 			return false;
 
-		if (!mpdclient_cmd_move(&c, range.start_index,
-					range.end_index))
+		if (!c.RunMove(range.start_index, range.end_index))
 			return true;
 
 		lw.selected++;
