@@ -32,9 +32,6 @@
 #include "FileListPage.hxx"
 #include "filelist.hxx"
 #include "util/Macros.hxx"
-#include "util/ScopeExit.hxx"
-
-#include <glib.h>
 
 #include <string.h>
 
@@ -128,10 +125,10 @@ private:
 
 public:
 	/* virtual methods from class Page */
-	void Paint() const override;
-	void Update(struct mpdclient &c, unsigned events) override;
+	void Paint() const noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 };
 
 /* search info */
@@ -139,14 +136,13 @@ class SearchHelpText final : public ListText {
 public:
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned idx) const override {
+				    unsigned idx) const noexcept override {
 		assert(idx < ARRAY_SIZE(help_text));
 
 		if (idx == 0) {
 			snprintf(buffer, size,
 				 " %s : %s",
-				 GetGlobalKeyBindings().GetKeyNames(Command::SCREEN_SEARCH,
-								    true),
+				 GetGlobalKeyBindings().GetKeyNames(Command::SCREEN_SEARCH).c_str(),
 				 "New search");
 			return buffer;
 		}
@@ -154,8 +150,7 @@ public:
 		if (idx == 1) {
 			snprintf(buffer, size,
 				 " %s : %s [%s]",
-				 GetGlobalKeyBindings().GetKeyNames(Command::SEARCH_MODE,
-								    true),
+				 GetGlobalKeyBindings().GetKeyNames(Command::SEARCH_MODE).c_str(),
 				 get_key_description(Command::SEARCH_MODE),
 				 gettext(mode[options.search_mode].label));
 			return buffer;
@@ -237,8 +232,7 @@ search_advanced_query(struct mpd_connection *connection, const char *query)
 	if (strchr(query, ':') == nullptr)
 		return nullptr;
 
-	char *str = g_strdup(query);
-	AtScopeExit(str) { g_free(str); };
+	std::string str(query);
 
 	static constexpr size_t N = 10;
 
@@ -263,8 +257,8 @@ search_advanced_query(struct mpd_connection *connection, const char *query)
 			if (spi != -1)
 				str[spi] = '\0';
 
-			matchv[n] = str + i + 1;
-			tabv[n] = str + spi + 1;
+			matchv[n] = &str[i + 1];
+			tabv[n] = &str[spi + 1];
 			table[n] = search_get_tag_id(tabv[n]);
 			if (table[n] < 0) {
 				screen_status_printf(_("Bad search tag %s"),
@@ -390,7 +384,7 @@ screen_search_init(ScreenManager &_screen, WINDOW *w, Size size)
 }
 
 void
-SearchPage::Paint() const
+SearchPage::Paint() const noexcept
 {
 	if (filelist) {
 		FileListPage::Paint();
@@ -400,7 +394,7 @@ SearchPage::Paint() const
 }
 
 const char *
-SearchPage::GetTitle(char *str, size_t size) const
+SearchPage::GetTitle(char *str, size_t size) const noexcept
 {
 	if (advanced_search_mode && !pattern.empty())
 		snprintf(str, size, "%s '%s'", _("Search"), pattern.c_str());
@@ -417,7 +411,7 @@ SearchPage::GetTitle(char *str, size_t size) const
 }
 
 void
-SearchPage::Update(struct mpdclient &c, unsigned events)
+SearchPage::Update(struct mpdclient &c, unsigned events) noexcept
 {
 	if (filelist != nullptr && events & MPD_IDLE_QUEUE) {
 		screen_browser_sync_highlights(filelist, &c.playlist);
