@@ -34,12 +34,10 @@
 #include "util/LocaleString.hxx"
 #include "util/Macros.hxx"
 #include "util/StringStrip.hxx"
-#include "util/StringUTF8.hxx"
 
 #include <mpd/client.h>
 
-#include <glib/gprintf.h>
-
+#include <algorithm>
 #include <vector>
 #include <string>
 
@@ -60,7 +58,7 @@ struct tag_label {
 	const char *label;
 };
 
-static const struct tag_label tag_labels[] = {
+static constexpr struct tag_label tag_labels[] = {
 	{ MPD_TAG_ARTIST, N_("Artist") },
 	{ MPD_TAG_TITLE, N_("Title") },
 	{ MPD_TAG_ALBUM, N_("Album") },
@@ -91,7 +89,7 @@ enum stats_label {
 	STATS_DBPLAYTIME,
 };
 
-static const char *const stats_labels[] = {
+static constexpr const char *stats_labels[] = {
 	N_("Number of artists"),
 	N_("Number of albums"),
 	N_("Number of songs"),
@@ -114,50 +112,52 @@ class SongPage final : public ListPage, ListText {
 	std::vector<std::string> lines;
 
 public:
-	SongPage(ScreenManager &_screen, WINDOW *w, Size size)
+	SongPage(ScreenManager &_screen, WINDOW *w, Size size) noexcept
 		:ListPage(w, size),
 		 screen(_screen) {
 		lw.hide_cursor = true;
 	}
 
-	~SongPage() override {
+	~SongPage() noexcept override {
 		Clear();
 	}
 
 private:
-	void Clear();
+	void Clear() noexcept;
 
 	/**
 	 * Appends a line with a fixed width for the label column.
 	 * Handles nullptr strings gracefully.
 	 */
 	void AppendLine(const char *label, const char *value,
-			unsigned label_col);
+			unsigned label_col) noexcept;
 
-	void AppendTag(const struct mpd_song *song, enum mpd_tag_type tag);
-	void AddSong(const struct mpd_song *song);
-	void AppendStatsLine(enum stats_label label, const char *value);
-	bool AddStats(struct mpd_connection *connection);
+	void AppendTag(const struct mpd_song *song,
+		       enum mpd_tag_type tag) noexcept;
+	void AddSong(const struct mpd_song *song) noexcept;
+	void AppendStatsLine(enum stats_label label,
+			     const char *value) noexcept;
+	bool AddStats(struct mpd_connection *connection) noexcept;
 
 public:
 	/* virtual methods from class Page */
-	void OnClose() override {
+	void OnClose() noexcept override {
 		Clear();
 	}
 
-	void Paint() const override;
-	void Update(struct mpdclient &c, unsigned events) override;
+	void Paint() const noexcept override;
+	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 
 private:
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const override;
+				    unsigned i) const noexcept override;
 };
 
 void
-SongPage::Clear()
+SongPage::Clear() noexcept
 {
 	lines.clear();
 
@@ -172,13 +172,13 @@ SongPage::Clear()
 }
 
 const char *
-SongPage::GetListItemText(char *, size_t, unsigned idx) const
+SongPage::GetListItemText(char *, size_t, unsigned idx) const noexcept
 {
 	return lines[idx].c_str();
 }
 
 static std::unique_ptr<Page>
-screen_song_init(ScreenManager &_screen, WINDOW *w, Size size)
+screen_song_init(ScreenManager &_screen, WINDOW *w, Size size) noexcept
 {
 	for (unsigned i = 0; tag_labels[i].label != nullptr; ++i) {
 		unsigned width = StringWidthMB(gettext(tag_labels[i].label));
@@ -199,78 +199,77 @@ screen_song_init(ScreenManager &_screen, WINDOW *w, Size size)
 }
 
 const char *
-SongPage::GetTitle(gcc_unused char *str, gcc_unused size_t size) const
+SongPage::GetTitle(gcc_unused char *str, gcc_unused size_t size) const noexcept
 {
 	return _("Song viewer");
 }
 
 void
-SongPage::Paint() const
+SongPage::Paint() const noexcept
 {
 	lw.Paint(TextListRenderer(*this));
 }
 
 void
-SongPage::AppendLine(const char *label, const char *value, unsigned label_col)
+SongPage::AppendLine(const char *label, const char *value_utf8,
+		     unsigned label_col) noexcept
 {
-	const unsigned label_width = StringWidthMB(label) + 2;
-
 	assert(label != nullptr);
-	assert(value != nullptr);
-	assert(g_utf8_validate(value, -1, nullptr));
+	assert(value_utf8 != nullptr);
+
+	static constexpr size_t BUFFER_SIZE = 1024;
+	if (label_col >= BUFFER_SIZE - 16)
+		return;
 
 	/* +2 for ': ' */
 	label_col += 2;
 	const int value_col = lw.size.width - label_col;
 	/* calculate the number of required linebreaks */
-	const gchar *value_iter = value;
-	const size_t label_length = strlen(label);
-	const size_t label_size = label_length + label_col;
+	const Utf8ToLocale value_locale(value_utf8);
+	const char *value = value_locale.c_str();
+
+	const char *const value_end = value + strlen(value);
+	const char *value_iter = value;
 
 	while (*value_iter != 0) {
-		char *entry = (char *)g_malloc(label_size), *entry_iter;
+		char buffer[BUFFER_SIZE];
+		const char *const buffer_end = buffer + BUFFER_SIZE;
+
+		char *p = buffer;
+		size_t n_space = label_col;
 		if (value_iter == value) {
-			memcpy(entry, label, label_length);
-			entry_iter = entry + label_length;
-			*entry_iter++ = ':';
+			const size_t label_length = std::min(strlen(label),
+							     BUFFER_SIZE - 16);
+			p = std::copy_n(label, label_length, p);
+			*p++ = ':';
 			/* fill the label column with whitespaces */
-			size_t n_space = label_col - label_width + 1;
-			memset(entry_iter, ' ', n_space);
-			entry_iter += n_space;
-		}
-		else {
-			/* fill the label column with whitespaces */
-			memset(entry, ' ', label_col);
-			entry_iter = entry + label_col;
+			const unsigned label_width = StringWidthMB(label);
+			n_space -= label_width + 1;
 		}
+
+		p = std::fill_n(p, n_space, ' ');
+
 		/* skip whitespaces */
 		value_iter = StripLeft(value_iter);
 
-		char *p = g_strdup(value_iter);
-		unsigned width = utf8_cut_width(p, value_col);
-		if (width == 0) {
+		const char *value_iter_end = AtWidthMB(value_iter,
+						       value_end - value_iter,
+						       value_col);
+		if (value_iter_end == value_iter)
 			/* not enough room for anything - bail out */
-			g_free(entry);
-			g_free(p);
 			break;
-		}
 
-		*entry_iter = 0;
+		p += snprintf(p, buffer_end - p, "%.*s",
+			      int(value_iter_end - value_iter), value_iter);
+		value_iter = value_iter_end;
 
-		value_iter += strlen(p);
-		p = replace_utf8_to_locale(p);
-		char *q = g_strconcat(entry, p, nullptr);
-		g_free(entry);
-		g_free(p);
-
-		lines.emplace_back(q);
-		g_free(q);
+		lines.emplace_back(buffer, p);
 	}
 }
 
 gcc_pure
 static const char *
-get_tag_label(unsigned tag)
+get_tag_label(unsigned tag) noexcept
 {
 	for (unsigned i = 0; tag_labels[i].label != nullptr; ++i)
 		if (tag_labels[i].tag_type == tag)
@@ -281,7 +280,8 @@ get_tag_label(unsigned tag)
 }
 
 void
-SongPage::AppendTag(const struct mpd_song *song, enum mpd_tag_type tag)
+SongPage::AppendTag(const struct mpd_song *song,
+		    enum mpd_tag_type tag) noexcept
 {
 	const char *label = get_tag_label(tag);
 	unsigned i = 0;
@@ -295,7 +295,7 @@ SongPage::AppendTag(const struct mpd_song *song, enum mpd_tag_type tag)
 }
 
 void
-SongPage::AddSong(const struct mpd_song *song)
+SongPage::AddSong(const struct mpd_song *song) noexcept
 {
 	assert(song != nullptr);
 
@@ -355,13 +355,13 @@ SongPage::AddSong(const struct mpd_song *song)
 }
 
 void
-SongPage::AppendStatsLine(enum stats_label label, const char *value)
+SongPage::AppendStatsLine(enum stats_label label, const char *value) noexcept
 {
 	AppendLine(gettext(stats_labels[label]), value, max_stats_label_width);
 }
 
 bool
-SongPage::AddStats(struct mpd_connection *connection)
+SongPage::AddStats(struct mpd_connection *connection) noexcept
 {
 	struct mpd_stats *mpd_stats = mpd_run_stats(connection);
 	if (mpd_stats == nullptr)
@@ -402,7 +402,7 @@ SongPage::AddStats(struct mpd_connection *connection)
 
 static void
 audio_format_to_string(char *buffer, size_t size,
-		       const struct mpd_audio_format *format)
+		       const struct mpd_audio_format *format) noexcept
 {
 #if LIBMPDCLIENT_CHECK_VERSION(2,10,0)
 	if (format->bits == MPD_SAMPLE_FORMAT_FLOAT) {
@@ -436,7 +436,7 @@ audio_format_to_string(char *buffer, size_t size,
 }
 
 void
-SongPage::Update(struct mpdclient &c, unsigned)
+SongPage::Update(struct mpdclient &c, unsigned) noexcept
 {
 	lines.clear();
 
@@ -447,21 +447,22 @@ SongPage::Update(struct mpdclient &c, unsigned)
 		next_song = nullptr;
 	}
 
+	const auto *const playing_song = c.GetPlayingSong();
+
 	if (selected_song != nullptr &&
-	    (c.song == nullptr ||
+	    (playing_song == nullptr ||
 	     strcmp(mpd_song_get_uri(selected_song),
-		    mpd_song_get_uri(c.song)) != 0 ||
-	     !c.playing_or_paused)) {
+		    mpd_song_get_uri(playing_song)) != 0)) {
 		lines.emplace_back(_("Selected song"));
 		AddSong(selected_song);
 		lines.emplace_back(std::string());
 	}
 
-	if (c.song != nullptr && c.playing_or_paused) {
+	if (playing_song) {
 		if (played_song != nullptr)
 			mpd_song_free(played_song);
 
-		played_song = mpd_song_dup(c.song);
+		played_song = mpd_song_dup(playing_song);
 		lines.emplace_back(_("Currently playing song"));
 		AddSong(played_song);
 
@@ -539,7 +540,7 @@ SongPage::OnCommand(struct mpdclient &c, Command cmd)
 		break;
 	}
 
-	if (screen_find(screen, &lw, cmd, *this)) {
+	if (screen_find(screen, lw, cmd, *this)) {
 		/* center the row */
 		lw.Center(lw.selected);
 		SetDirty();
@@ -558,7 +559,7 @@ const PageMeta screen_song = {
 
 void
 screen_song_switch(ScreenManager &_screen, struct mpdclient &c,
-		   const struct mpd_song &song)
+		   const struct mpd_song &song) noexcept
 {
 	next_song = mpd_song_dup(&song);
 	_screen.Switch(screen_song, c);
