@@ -18,27 +18,27 @@
  */
 
 #include "StatusBar.hxx"
-#include "Event.hxx"
 #include "Options.hxx"
 #include "Styles.hxx"
 #include "i18n.h"
 #include "strfsong.hxx"
-#include "player_command.hxx"
+#include "DelayedSeek.hxx"
 #include "time_format.hxx"
-#include "util/StringUTF8.hxx"
+#include "util/LocaleString.hxx"
 
 #include <mpd/client.h>
 
 #include <assert.h>
 #include <string.h>
 
-StatusBar::StatusBar(Point p, unsigned width)
-	:window(p, {width, 1u})
+StatusBar::StatusBar(boost::asio::io_service &io_service,
+		     Point p, unsigned width) noexcept
+	:window(p, {width, 1u}),
+	 message_timer(io_service)
 #ifndef NCMPC_MINI
-	, hscroll(window.w, options.scroll_sep.c_str())
+	, hscroll(io_service, window.w, options.scroll_sep.c_str())
 #endif
 {
-
 	leaveok(window.w, false);
 	keypad(window.w, true);
 
@@ -48,7 +48,7 @@ StatusBar::StatusBar(Point p, unsigned width)
 #endif
 }
 
-StatusBar::~StatusBar()
+StatusBar::~StatusBar() noexcept
 {
 #ifndef NCMPC_MINI
 	if (options.scroll)
@@ -57,24 +57,20 @@ StatusBar::~StatusBar()
 }
 
 void
-StatusBar::ClearMessage()
+StatusBar::ClearMessage() noexcept
 {
-	if (message_source_id != 0) {
-		g_source_remove(message_source_id);
-		message_source_id = 0;
-	}
+	message_timer.cancel();
+	message.clear();
 
-	WINDOW *w = window.w;
-
-	wmove(w, 0, 0);
-	wclrtoeol(w);
-	wrefresh(w);
+	Paint();
+	doupdate();
 }
 
 #ifndef NCMPC_MINI
 
 static void
-format_bitrate(char *p, size_t max_length, const struct mpd_status *status)
+format_bitrate(char *p, size_t max_length,
+	       const struct mpd_status *status) noexcept
 {
 	if (options.visible_bitrate && mpd_status_get_kbit_rate(status) > 0)
 		snprintf(p, max_length,
@@ -88,7 +84,8 @@ format_bitrate(char *p, size_t max_length, const struct mpd_status *status)
 
 void
 StatusBar::Update(const struct mpd_status *status,
-		  const struct mpd_song *song)
+		  const struct mpd_song *song,
+		  const DelayedSeek &seek) noexcept
 {
 	const auto state = status == nullptr
 		? MPD_STATE_UNKNOWN
@@ -110,13 +107,12 @@ StatusBar::Update(const struct mpd_status *status,
 	}
 
 	left_width = left_text != nullptr
-		? utf8_width(left_text) + 1
+		? StringWidthMB(left_text) + 1
 		: 0;
 
 	if (state == MPD_STATE_PLAY || state == MPD_STATE_PAUSE) {
-		unsigned elapsed_time = seek_id >= 0 &&
-			seek_id == mpd_status_get_song_id(status)
-			? (unsigned)seek_target_time
+		unsigned elapsed_time = seek.IsSeeking(mpd_status_get_song_id(status))
+			? seek.GetTime()
 			: mpd_status_get_elapsed_time(status);
 		const unsigned total_time = mpd_status_get_total_time(status);
 
@@ -158,7 +154,7 @@ StatusBar::Update(const struct mpd_status *status,
 #endif
 		}
 
-		right_width = utf8_width(right_text);
+		right_width = StringWidthMB(right_text);
 
 #ifndef NCMPC_MINI
 		int width = COLS - left_width - right_width;
@@ -174,9 +170,8 @@ StatusBar::Update(const struct mpd_status *status,
 
 		/* scroll if the song name is to long */
 #ifndef NCMPC_MINI
-		center_width = utf8_width(center_text.c_str());
-		if (options.scroll &&
-		    utf8_width(center_text.c_str()) > (unsigned)width) {
+		center_width = StringWidthMB(center_text.c_str());
+		if (options.scroll && center_width > (unsigned)width) {
 			hscroll.Set(left_width, 0, width, center_text.c_str(),
 				    Style::STATUS);
 		} else {
@@ -197,15 +192,20 @@ StatusBar::Update(const struct mpd_status *status,
 }
 
 void
-StatusBar::Paint() const
+StatusBar::Paint() const noexcept
 {
 	WINDOW *w = window.w;
 
-	if (message_source_id != 0)
-		return;
-
 	wmove(w, 0, 0);
 	wclrtoeol(w);
+
+	if (!message.empty()) {
+		SelectStyle(w, Style::STATUS_ALERT);
+		waddstr(w, message.c_str());
+		wnoutrefresh(w);
+		return;
+	}
+
 	SelectStyle(w, Style::STATUS_BOLD);
 
 	if (left_text != nullptr)
@@ -242,40 +242,27 @@ StatusBar::Paint() const
 }
 
 void
-StatusBar::OnResize(Point p, unsigned width)
+StatusBar::OnResize(Point p, unsigned width) noexcept
 {
 	window.Resize({width, 1u});
 	window.Move(p);
 }
 
-inline bool
-StatusBar::OnClearMessageTimer()
-{
-	assert(message_source_id != 0);
-	message_source_id = 0;
-	ClearMessage();
-	return false;
-}
-
 void
-StatusBar::SetMessage(const char *msg)
+StatusBar::SetMessage(const char *msg) noexcept
 {
-	WINDOW *w = window.w;
-
 #ifndef NCMPC_MINI
 	if (options.scroll)
 		hscroll.Clear();
 #endif
 
-	wmove(w, 0, 0);
-	wclrtoeol(w);
-	SelectStyle(w, Style::STATUS_ALERT);
-	waddstr(w, msg);
-	wnoutrefresh(w);
+	message = msg;
+	Paint();
+	doupdate();
 
-	if (message_source_id != 0)
-		g_source_remove(message_source_id);
-	message_source_id = ScheduleTimeout<StatusBar,
-					    &StatusBar::OnClearMessageTimer>(options.status_message_time,
-									     *this);
+	boost::system::error_code error;
+	message_timer.expires_from_now(options.status_message_time,
+				       error);
+	message_timer.async_wait(std::bind(&StatusBar::OnMessageTimer, this,
+					   std::placeholders::_1));
 }
