@@ -22,15 +22,13 @@
 #include "screen_find.hxx"
 #include "charset.hxx"
 
-#include <glib.h>
-
 #include <algorithm>
 
 #include <assert.h>
 #include <string.h>
 
 void
-TextPage::Clear()
+TextPage::Clear() noexcept
 {
 	lw.Reset();
 	lines.clear();
@@ -38,7 +36,7 @@ TextPage::Clear()
 }
 
 void
-TextPage::Append(const char *str)
+TextPage::Append(const char *str) noexcept
 {
 	assert(str != nullptr);
 
@@ -72,16 +70,15 @@ TextPage::Append(const char *str)
 }
 
 const char *
-TextPage::GetListItemText(char *buffer, size_t size, unsigned idx) const
+TextPage::GetListItemText(char *buffer, size_t size, unsigned idx) const noexcept
 {
 	assert(idx < lines.size());
 
-	g_strlcpy(buffer, Utf8ToLocale(lines[idx].c_str()).c_str(), size);
-	return buffer;
+	return utf8_to_locale(lines[idx].c_str(), buffer, size);
 }
 
 void
-TextPage::Paint() const
+TextPage::Paint() const noexcept
 {
 	lw.Paint(TextListRenderer(*this));
 }
@@ -93,7 +90,7 @@ TextPage::OnCommand(struct mpdclient &c, Command cmd)
 		return true;
 
 	lw.SetCursor(lw.start);
-	if (screen_find(screen, &lw, cmd, *this)) {
+	if (screen_find(screen, lw, cmd, *this)) {
 		/* center the row */
 		lw.Center(lw.selected);
 		SetDirty();
