@@ -74,6 +74,7 @@
 #define CONF_SEARCH_MODE "search-mode"
 #define CONF_HIDE_CURSOR "hide-cursor"
 #define CONF_SEEK_TIME "seek-time"
+#define CONF_LIBRARY_PAGE_TAGS "library-page-tags"
 #define CONF_SCREEN_LIST "screen-list"
 #define CONF_TIMEDISPLAY_TYPE "timedisplay-type"
 #define CONF_HOST "host"
@@ -103,15 +104,16 @@
 #define KEYS_FILENAME "keys"
 #endif
 
+gcc_pure
 static bool
-str2bool(char *str)
+str2bool(char *str) noexcept
 {
 	return strcasecmp(str, "yes") == 0 || strcasecmp(str, "true") == 0 ||
 		strcasecmp(str, "on") == 0 || strcasecmp(str, "1") == 0;
 }
 
 static void
-print_error(const char *msg, const char *input)
+print_error(const char *msg, const char *input) noexcept
 {
 	fprintf(stderr, "%s: %s ('%s')\n",
 		/* To translators: prefix for error messages */
@@ -120,7 +122,7 @@ print_error(const char *msg, const char *input)
 
 gcc_const
 static bool
-is_word_char(char ch)
+is_word_char(char ch) noexcept
 {
 	return IsAlphaNumericASCII(ch) || ch == '-' || ch == '_';
 }
@@ -236,7 +238,7 @@ separate_value(char *p)
 		return nullptr;
 	}
 
-	*value++ = 0;
+	*StripRight(p, value++) = '\0';
 
 	return Strip(value);
 }
@@ -257,7 +259,7 @@ parse_color(char *str)
  * separated values.
  */
 static char *
-after_comma(char *p)
+after_comma(char *p) noexcept
 {
 	char *comma = strchr(p, ',');
 
@@ -316,7 +318,7 @@ parse_color_definition(char *str)
 #endif
 
 static std::string
-GetStringValue(const char *s)
+GetStringValue(const char *s) noexcept
 {
 	size_t length = strlen(s);
 
@@ -377,6 +379,32 @@ check_screen_list(char *value)
 	return screen;
 }
 
+#ifdef ENABLE_LIBRARY_PAGE
+
+static auto
+ParseTagList(char *value)
+{
+	std::vector<enum mpd_tag_type> result;
+
+	while (char *name = NextItem(value)) {
+		auto type = mpd_tag_name_iparse(name);
+		if (type == MPD_TAG_UNKNOWN) {
+			print_error(_("Unknown MPD tag"), name);
+			result.clear();
+			return result;
+		}
+
+		result.emplace_back(type);
+	}
+
+	if (result.empty())
+		fprintf(stderr, "Empty tag list\n");
+
+	return result;
+}
+
+#endif
+
 static int
 get_search_mode(char *value)
 {
@@ -522,7 +550,13 @@ parse_line(char *line)
 		options.hide_cursor = std::chrono::seconds(atoi(value));
 	else if (!strcasecmp(CONF_SEEK_TIME, name))
 		options.seek_time = atoi(value);
-	else if (!strcasecmp(CONF_SCREEN_LIST, name)) {
+	else if (!strcasecmp(CONF_LIBRARY_PAGE_TAGS, name)) {
+#ifdef ENABLE_LIBRARY_PAGE
+		options.library_page_tags = ParseTagList(value);
+		if (options.library_page_tags.empty())
+			return false;
+#endif
+	} else if (!strcasecmp(CONF_SCREEN_LIST, name)) {
 		options.screen_list = check_screen_list(value);
 	} else if (!strcasecmp(CONF_HOST, name))
 		options.host = GetStringValue(value);
