@@ -20,24 +20,26 @@
 #include "hscroll.hxx"
 #include "Styles.hxx"
 #include "charset.hxx"
-#include "Event.hxx"
 
 #include <algorithm>
 
 #include <assert.h>
 
-inline bool
-hscroll::TimerCallback()
+inline void
+hscroll::TimerCallback(const boost::system::error_code &error) noexcept
 {
+	if (error)
+		return;
+
 	Step();
 	Paint();
 	wrefresh(w);
-	return true;
+	ScheduleTimer();
 }
 
 void
 hscroll::Set(unsigned _x, unsigned _y, unsigned _width, const char *_text,
-	     Style _style, attr_t _attr)
+	     Style _style, attr_t _attr) noexcept
 {
 	assert(w != nullptr);
 	assert(_text != nullptr);
@@ -50,21 +52,18 @@ hscroll::Set(unsigned _x, unsigned _y, unsigned _width, const char *_text,
 	if (!basic.Set(_width, _text))
 		return;
 
-	if (source_id == 0)
-		source_id = ScheduleTimeout<hscroll, &hscroll::TimerCallback>(std::chrono::seconds(1), *this);
+	ScheduleTimer();
 }
 
 void
-hscroll::Clear()
+hscroll::Clear() noexcept
 {
 	basic.Clear();
-
-	if (source_id != 0)
-		g_source_remove(std::exchange(source_id, 0));
+	timer.cancel();
 }
 
 void
-hscroll::Paint() const
+hscroll::Paint() const noexcept
 {
 	assert(w != nullptr);
 	assert(basic.IsDefined());
