@@ -24,72 +24,49 @@
 #include "gidle.hxx"
 #include "charset.hxx"
 
-#ifdef ENABLE_ASYNC_CONNECT
-#include "aconnect.hxx"
-#endif
-
 #include <mpd/client.h>
 
-#include <glib.h>
-
 #include <assert.h>
 
-static gboolean
-mpdclient_enter_idle_callback(gpointer user_data)
+void
+mpdclient::OnEnterIdleTimer(const boost::system::error_code &error) noexcept
 {
-	auto *c = (struct mpdclient *)user_data;
-	assert(c->enter_idle_source_id != 0);
-	assert(c->source != nullptr);
-	assert(!c->idle);
+	if (error)
+		return;
 
-	c->enter_idle_source_id = 0;
-	c->idle = c->source->Enter();
-	return false;
+	assert(source != nullptr);
+	assert(!idle);
+
+	idle = source->Enter();
 }
 
-static void
-mpdclient_schedule_enter_idle(struct mpdclient *c)
+void
+mpdclient::ScheduleEnterIdle() noexcept
 {
-	assert(c != nullptr);
-	assert(c->source != nullptr);
+	assert(source != nullptr);
 
-	if (c->enter_idle_source_id == 0)
-		/* automatically re-enter MPD "idle" mode */
-		c->enter_idle_source_id =
-			g_idle_add(mpdclient_enter_idle_callback, c);
-}
-
-static void
-mpdclient_cancel_enter_idle(struct mpdclient *c)
-{
-	if (c->enter_idle_source_id != 0) {
-		g_source_remove(c->enter_idle_source_id);
-		c->enter_idle_source_id = 0;
-	}
+	/* automatically re-enter MPD "idle" mode */
+	boost::system::error_code error;
+	enter_idle_timer.expires_from_now(std::chrono::seconds(0), error);
+	enter_idle_timer.async_wait(std::bind(&mpdclient::OnEnterIdleTimer,
+					      this, std::placeholders::_1));
 }
 
 static void
 mpdclient_invoke_error_callback(enum mpd_error error,
 				const char *message)
 {
-	char *allocated;
 	if (error == MPD_ERROR_SERVER)
 		/* server errors are UTF-8, the others are locale */
-		message = allocated = utf8_to_locale(message);
+		mpdclient_error_callback(Utf8ToLocale(message).c_str());
 	else
-		allocated = nullptr;
-
-	mpdclient_error_callback(message);
-	g_free(allocated);
+		mpdclient_error_callback(message);
 }
 
-static void
-mpdclient_invoke_error_callback1(struct mpdclient *c)
+void
+mpdclient::InvokeErrorCallback() noexcept
 {
-	assert(c != nullptr);
-	assert(c->connection != nullptr);
-
-	struct mpd_connection *connection = c->connection;
+	assert(connection != nullptr);
 
 	enum mpd_error error = mpd_connection_get_error(connection);
 	assert(error != MPD_ERROR_SUCCESS);
@@ -98,34 +75,35 @@ mpdclient_invoke_error_callback1(struct mpdclient *c)
 					mpd_connection_get_error_message(connection));
 }
 
-static void
-mpdclient_gidle_callback(enum mpd_error error,
-			 gcc_unused enum mpd_server_error server_error,
-			 const char *message, unsigned events,
-			 void *ctx)
+void
+mpdclient::OnIdle(unsigned _events) noexcept
 {
-	auto *c = (struct mpdclient *)ctx;
+	assert(IsConnected());
 
-	c->idle = false;
+	idle = false;
 
-	assert(c->IsConnected());
+	events |= _events;
+	Update();
 
-	if (error != MPD_ERROR_SUCCESS) {
-		mpdclient_invoke_error_callback(error, message);
-		c->Disconnect();
-		mpdclient_lost_callback();
-		return;
-	}
+	mpdclient_idle_callback(events);
+	events = 0;
 
-	c->events |= events;
-	c->Update();
+	if (source != nullptr)
+		ScheduleEnterIdle();
+}
 
-	mpdclient_idle_callback(c->events);
+void
+mpdclient::OnIdleError(enum mpd_error error,
+		       gcc_unused enum mpd_server_error server_error,
+		       const char *message) noexcept
+{
+	assert(IsConnected());
 
-	c->events = 0;
+	idle = false;
 
-	if (c->source != nullptr)
-		mpdclient_schedule_enter_idle(c);
+	mpdclient_invoke_error_callback(error, message);
+	Disconnect();
+	mpdclient_lost_callback();
 }
 
 /****************************************************************************/
@@ -174,15 +152,17 @@ settings_is_local_socket(const struct mpd_settings *settings)
 #endif
 #endif
 
-mpdclient::mpdclient(const char *_host, unsigned _port,
+mpdclient::mpdclient(boost::asio::io_service &io_service,
+		     const char *_host, unsigned _port,
 		     unsigned _timeout_ms, const char *_password)
-	:timeout_ms(_timeout_ms), password(_password)
+	:timeout_ms(_timeout_ms), password(_password),
+	 enter_idle_timer(io_service)
 {
 #ifdef ENABLE_ASYNC_CONNECT
 	settings = mpd_settings_new(_host, _port, _timeout_ms,
 				    nullptr, nullptr);
 	if (settings == nullptr)
-		g_error("Out of memory");
+		fprintf(stderr, "Out of memory\n");
 
 #ifndef _WIN32
 	settings2 = _host == nullptr && _port == 0 &&
@@ -235,19 +215,19 @@ mpdclient::GetSettingsName() const
 #endif
 }
 
-static void
-mpdclient_status_free(struct mpdclient *c)
+void
+mpdclient::ClearStatus() noexcept
 {
-	if (c->status == nullptr)
+	if (status == nullptr)
 		return;
 
-	mpd_status_free(c->status);
-	c->status = nullptr;
+	mpd_status_free(status);
+	status = nullptr;
 
-	c->volume = -1;
-	c->playing = false;
-	c->playing_or_paused = false;
-	c->state = MPD_STATE_UNKNOWN;
+	volume = -1;
+	playing = false;
+	playing_or_paused = false;
+	state = MPD_STATE_UNKNOWN;
 }
 
 void
@@ -260,7 +240,7 @@ mpdclient::Disconnect()
 	}
 #endif
 
-	mpdclient_cancel_enter_idle(this);
+	CancelEnterIdle();
 
 	delete source;
 	source = nullptr;
@@ -272,53 +252,50 @@ mpdclient::Disconnect()
 	}
 	connection = nullptr;
 
-	mpdclient_status_free(this);
+	ClearStatus();
 
 	playlist.clear();
 
-	if (song)
-		song = nullptr;
+	current_song = nullptr;
 
 	/* everything has changed after a disconnect */
 	events |= MPD_IDLE_ALL;
 }
 
-static bool
-mpdclient_connected(struct mpdclient *c,
-		    struct mpd_connection *connection)
+bool
+mpdclient::OnConnected(struct mpd_connection *_connection) noexcept
 {
-	c->connection = connection;
+	connection = _connection;
 
 	if (mpd_connection_get_error(connection) != MPD_ERROR_SUCCESS) {
-		mpdclient_invoke_error_callback1(c);
-		c->Disconnect();
+		InvokeErrorCallback();
+		Disconnect();
 		mpdclient_failed_callback();
 		return false;
 	}
 
 #ifdef ENABLE_ASYNC_CONNECT
-	if (c->timeout_ms > 0)
-		mpd_connection_set_timeout(connection, c->timeout_ms);
+	if (timeout_ms > 0)
+		mpd_connection_set_timeout(connection, timeout_ms);
 #endif
 
 	/* send password */
-	if (c->password != nullptr &&
-	    !mpd_run_password(connection, c->password)) {
-		mpdclient_invoke_error_callback1(c);
-		c->Disconnect();
+	if (password != nullptr &&
+	    !mpd_run_password(connection, password)) {
+		InvokeErrorCallback();
+		Disconnect();
 		mpdclient_failed_callback();
 		return false;
 	}
 
-	c->source = new MpdIdleSource(*connection,
-				      mpdclient_gidle_callback, c);
-	mpdclient_schedule_enter_idle(c);
+	source = new MpdIdleSource(get_io_service(), *connection, *this);
+	ScheduleEnterIdle();
 
-	++c->connection_id;
+	++connection_id;
 
 	/* everything has changed after a connection has been
 	   established */
-	c->events = (enum mpd_idle)MPD_IDLE_ALL;
+	events = (enum mpd_idle)MPD_IDLE_ALL;
 
 	mpdclient_connected_callback();
 	return true;
@@ -326,51 +303,34 @@ mpdclient_connected(struct mpdclient *c,
 
 #ifdef ENABLE_ASYNC_CONNECT
 
-static void
-mpdclient_aconnect_start(struct mpdclient *c,
-			 const struct mpd_settings *settings);
-
-static const struct mpd_settings *
-mpdclient_get_settings(const struct mpdclient *c)
+void
+mpdclient::OnAsyncMpdConnect(struct mpd_connection *_connection) noexcept
 {
-#ifndef _WIN32
-	if (c->connecting2)
-		return c->settings2;
-#endif
+	assert(async_connect != nullptr);
+	async_connect = nullptr;
 
-	return c->settings;
-}
-
-static void
-mpdclient_connect_success(struct mpd_connection *connection, void *ctx)
-{
-	auto *c = (struct mpdclient *)ctx;
-	assert(c->async_connect != nullptr);
-	c->async_connect = nullptr;
-
-	const char *password =
-		mpd_settings_get_password(mpdclient_get_settings(c));
-	if (password != nullptr && !mpd_run_password(connection, password)) {
-		mpdclient_error_callback(mpd_connection_get_error_message(connection));
-		mpd_connection_free(connection);
+	const char *password2 =
+		mpd_settings_get_password(&GetSettings());
+	if (password2 != nullptr && !mpd_run_password(_connection, password2)) {
+		mpdclient_error_callback(mpd_connection_get_error_message(_connection));
+		mpd_connection_free(_connection);
 		mpdclient_failed_callback();
 		return;
 	}
 
-	mpdclient_connected(c, connection);
+	OnConnected(_connection);
 }
 
-static void
-mpdclient_connect_error(const char *message, void *ctx)
+void
+mpdclient::OnAsyncMpdConnectError(const char *message) noexcept
 {
-	auto *c = (struct mpdclient *)ctx;
-	assert(c->async_connect != nullptr);
-	c->async_connect = nullptr;
+	assert(async_connect != nullptr);
+	async_connect = nullptr;
 
 #ifndef _WIN32
-	if (!c->connecting2 && c->settings2 != nullptr) {
-		c->connecting2 = true;
-		mpdclient_aconnect_start(c, c->settings2);
+	if (!connecting2 && settings2 != nullptr) {
+		connecting2 = true;
+		StartConnect(*settings2);
 		return;
 	}
 #endif
@@ -379,19 +339,13 @@ mpdclient_connect_error(const char *message, void *ctx)
 	mpdclient_failed_callback();
 }
 
-static constexpr AsyncMpdConnectHandler mpdclient_connect_handler = {
-	.success = mpdclient_connect_success,
-	.error = mpdclient_connect_error,
-};
-
-static void
-mpdclient_aconnect_start(struct mpdclient *c,
-			 const struct mpd_settings *settings)
+void
+mpdclient::StartConnect(const struct mpd_settings &s) noexcept
 {
-	aconnect_start(&c->async_connect,
-		       mpd_settings_get_host(settings),
-		       mpd_settings_get_port(settings),
-		       mpdclient_connect_handler, c);
+	aconnect_start(get_io_service(), &async_connect,
+		       mpd_settings_get_host(&s),
+		       mpd_settings_get_port(&s),
+		       *this);
 }
 
 #endif
@@ -406,15 +360,15 @@ mpdclient::Connect()
 #ifndef _WIN32
 	connecting2 = false;
 #endif
-	mpdclient_aconnect_start(this, settings);
+	StartConnect(*settings);
 #else
 	/* connect to MPD */
 	struct mpd_connection *new_connection =
 		mpd_connection_new(host, port, timeout_ms);
 	if (new_connection == nullptr)
-		g_error("Out of memory");
+		fprintf(stderr, "Out of memory\n");
 
-	mpdclient_connected(this, new_connection);
+	OnConnected(new_connection);
 #endif
 }
 
@@ -427,7 +381,7 @@ mpdclient::Update()
 		return false;
 
 	/* free the old status */
-	mpdclient_status_free(this);
+	ClearStatus();
 
 	/* retrieve new status */
 	status = mpd_run_status(c);
@@ -453,9 +407,8 @@ mpdclient::Update()
 	}
 
 	/* update the current song */
-	if (song == nullptr || mpd_status_get_song_id(status) >= 0) {
-		song = playlist.GetChecked(mpd_status_get_song_pos(status));
-	}
+	if (current_song == nullptr || mpd_status_get_song_id(status) >= 0)
+		current_song = playlist.GetChecked(mpd_status_get_song_pos(status));
 
 	return true;
 }
@@ -468,26 +421,26 @@ mpdclient::GetConnection()
 		source->Leave();
 
 		if (source != nullptr)
-			mpdclient_schedule_enter_idle(this);
+			ScheduleEnterIdle();
 	}
 
 	return connection;
 }
 
-static struct mpd_status *
-mpdclient_recv_status(struct mpdclient *c)
+const struct mpd_status *
+mpdclient::ReceiveStatus() noexcept
 {
-	assert(c->connection != nullptr);
+	assert(connection != nullptr);
 
-	struct mpd_status *status = mpd_recv_status(c->connection);
-	if (status == nullptr) {
-		c->HandleError();
+	struct mpd_status *new_status = mpd_recv_status(connection);
+	if (new_status == nullptr) {
+		HandleError();
 		return nullptr;
 	}
 
-	if (c->status != nullptr)
-		mpd_status_free(c->status);
-	return c->status = status;
+	if (status != nullptr)
+		mpd_status_free(status);
+	return status = new_status;
 }
 
 /****************************************************************************/
@@ -522,78 +475,70 @@ mpdclient_cmd_crop(struct mpdclient *c)
 }
 
 bool
-mpdclient_cmd_clear(struct mpdclient *c)
+mpdclient::RunClearQueue() noexcept
 {
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
+	auto *c = GetConnection();
+	if (c == nullptr)
 		return false;
 
 	/* send "clear" and "status" */
-	if (!mpd_command_list_begin(connection, false) ||
-	    !mpd_send_clear(connection) ||
-	    !mpd_send_status(connection) ||
-	    !mpd_command_list_end(connection))
-		return c->HandleError();
+	if (!mpd_command_list_begin(c, false) ||
+	    !mpd_send_clear(c) ||
+	    !mpd_send_status(c) ||
+	    !mpd_command_list_end(c))
+		return HandleError();
 
 	/* receive the new status, store it in the mpdclient struct */
 
-	struct mpd_status *status = mpdclient_recv_status(c);
-	if (status == nullptr)
+	const struct mpd_status *new_status = ReceiveStatus();
+	if (new_status == nullptr)
 		return false;
 
-	if (!mpd_response_finish(connection))
-		return c->HandleError();
+	if (!mpd_response_finish(c))
+		return HandleError();
 
 	/* update mpdclient.playlist */
 
-	if (mpd_status_get_queue_length(status) == 0) {
+	if (mpd_status_get_queue_length(new_status) == 0) {
 		/* after the "clear" command, the queue is really
 		   empty - this means we can clear it locally,
 		   reducing the UI latency */
-		c->playlist.clear();
-		c->playlist.version = mpd_status_get_queue_version(status);
-		c->song = nullptr;
+		playlist.clear();
+		playlist.version = mpd_status_get_queue_version(new_status);
+		current_song = nullptr;
 	}
 
-	c->events |= MPD_IDLE_QUEUE;
+	events |= MPD_IDLE_QUEUE;
 	return true;
 }
 
 bool
-mpdclient_cmd_volume(struct mpdclient *c, int value)
+mpdclient::RunVolume(unsigned value) noexcept
 {
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
+	struct mpd_connection *c = GetConnection();
+	if (c == nullptr)
 		return false;
 
-	mpd_send_set_volume(connection, value);
-	return c->FinishCommand();
+	mpd_send_set_volume(c, value);
+	return FinishCommand();
 }
 
 bool
-mpdclient_cmd_volume_up(struct mpdclient *c)
+mpdclient::RunVolumeUp() noexcept
 {
-	if (c->volume < 0 || c->volume >= 100)
+	if (volume < 0 || volume >= 100)
 		return true;
 
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
-		return false;
-
-	return mpdclient_cmd_volume(c, ++c->volume);
+	return RunVolume(++volume);
 }
 
 bool
-mpdclient_cmd_volume_down(struct mpdclient *c)
+mpdclient::RunVolumeDown() noexcept
 {
-	if (c->volume <= 0)
+	if (volume <= 0)
 		return true;
 
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
-		return false;
-
-	return mpdclient_cmd_volume(c, --c->volume);
+	return RunVolume(--volume);
 }
 
 bool
@@ -608,56 +553,52 @@ mpdclient_cmd_add_path(struct mpdclient *c, const char *path_utf8)
 }
 
 bool
-mpdclient_cmd_add(struct mpdclient *c, const struct mpd_song *song)
+mpdclient::RunAdd(const struct mpd_song &song) noexcept
 {
-	assert(c != nullptr);
-	assert(song != nullptr);
-
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr || c->status == nullptr)
+	auto *c = GetConnection();
+	if (c == nullptr || status == nullptr)
 		return false;
 
 	/* send the add command to mpd; at the same time, get the new
 	   status (to verify the new playlist id) and the last song
 	   (we hope that's the song we just added) */
 
-	if (!mpd_command_list_begin(connection, true) ||
-	    !mpd_send_add(connection, mpd_song_get_uri(song)) ||
-	    !mpd_send_status(connection) ||
-	    !mpd_send_get_queue_song_pos(connection,
-					 c->playlist.size()) ||
-	    !mpd_command_list_end(connection) ||
-	    !mpd_response_next(connection))
-		return c->HandleError();
+	if (!mpd_command_list_begin(c, true) ||
+	    !mpd_send_add(c, mpd_song_get_uri(&song)) ||
+	    !mpd_send_status(c) ||
+	    !mpd_send_get_queue_song_pos(c, playlist.size()) ||
+	    !mpd_command_list_end(c) ||
+	    !mpd_response_next(c))
+		return HandleError();
 
-	c->events |= MPD_IDLE_QUEUE;
+	events |= MPD_IDLE_QUEUE;
 
-	struct mpd_status *status = mpdclient_recv_status(c);
-	if (status == nullptr)
+	const struct mpd_status *new_status = ReceiveStatus();
+	if (new_status == nullptr)
 		return false;
 
-	if (!mpd_response_next(connection))
-		return c->HandleError();
+	if (!mpd_response_next(c))
+		return HandleError();
 
-	struct mpd_song *new_song = mpd_recv_song(connection);
-	if (!mpd_response_finish(connection) || new_song == nullptr) {
+	struct mpd_song *new_song = mpd_recv_song(c);
+	if (!mpd_response_finish(c) || new_song == nullptr) {
 		if (new_song != nullptr)
 			mpd_song_free(new_song);
 
-		return mpd_connection_clear_error(connection) ||
-			c->HandleError();
+		return mpd_connection_clear_error(c) ||
+			HandleError();
 	}
 
-	if (mpd_status_get_queue_length(status) == c->playlist.size() + 1 &&
-	    mpd_status_get_queue_version(status) == c->playlist.version + 1) {
+	if (mpd_status_get_queue_length(new_status) == playlist.size() + 1 &&
+	    mpd_status_get_queue_version(new_status) == playlist.version + 1) {
 		/* the cheap route: match on the new playlist length
 		   and its version, we can keep our local playlist
 		   copy in sync */
-		c->playlist.version = mpd_status_get_queue_version(status);
+		playlist.version = mpd_status_get_queue_version(new_status);
 
 		/* the song we just received has the correct id;
 		   append it to the local playlist */
-		c->playlist.push_back(*new_song);
+		playlist.push_back(*new_song);
 	}
 
 	mpd_song_free(new_song);
@@ -666,100 +607,99 @@ mpdclient_cmd_add(struct mpdclient *c, const struct mpd_song *song)
 }
 
 bool
-mpdclient_cmd_delete(struct mpdclient *c, int idx)
+mpdclient::RunDelete(unsigned pos) noexcept
 {
-	struct mpd_connection *connection = c->GetConnection();
-
-	if (connection == nullptr || c->status == nullptr)
+	auto *c = GetConnection();
+	if (c == nullptr || status == nullptr)
 		return false;
 
-	if (idx < 0 || (guint)idx >= c->playlist.size())
+	if (pos >= playlist.size())
 		return false;
 
-	const auto &song = c->playlist[idx];
+	const auto &song = playlist[pos];
 
 	/* send the delete command to mpd; at the same time, get the
 	   new status (to verify the playlist id) */
 
-	if (!mpd_command_list_begin(connection, false) ||
-	    !mpd_send_delete_id(connection, mpd_song_get_id(&song)) ||
-	    !mpd_send_status(connection) ||
-	    !mpd_command_list_end(connection))
-		return c->HandleError();
+	if (!mpd_command_list_begin(c, false) ||
+	    !mpd_send_delete_id(c, mpd_song_get_id(&song)) ||
+	    !mpd_send_status(c) ||
+	    !mpd_command_list_end(c))
+		return HandleError();
 
-	c->events |= MPD_IDLE_QUEUE;
+	events |= MPD_IDLE_QUEUE;
 
-	struct mpd_status *status = mpdclient_recv_status(c);
-	if (status == nullptr)
+	const struct mpd_status *new_status = ReceiveStatus();
+	if (new_status == nullptr)
 		return false;
 
-	if (!mpd_response_finish(connection))
-		return c->HandleError();
+	if (!mpd_response_finish(c))
+		return HandleError();
 
-	if (mpd_status_get_queue_length(status) == c->playlist.size() - 1 &&
-	    mpd_status_get_queue_version(status) == c->playlist.version + 1) {
+	if (mpd_status_get_queue_length(new_status) == playlist.size() - 1 &&
+	    mpd_status_get_queue_version(new_status) == playlist.version + 1) {
 		/* the cheap route: match on the new playlist length
 		   and its version, we can keep our local playlist
 		   copy in sync */
-		c->playlist.version = mpd_status_get_queue_version(status);
+		playlist.version = mpd_status_get_queue_version(new_status);
 
 		/* remove the song from the local playlist */
-		c->playlist.RemoveIndex(idx);
+		playlist.RemoveIndex(pos);
 
 		/* remove references to the song */
-		if (c->song == &song)
-			c->song = nullptr;
+		if (current_song == &song)
+			current_song = nullptr;
 	}
 
 	return true;
 }
 
 bool
-mpdclient_cmd_delete_range(struct mpdclient *c, unsigned start, unsigned end)
+mpdclient::RunDeleteRange(unsigned start, unsigned end) noexcept
 {
 	if (end == start + 1)
 		/* if that's not really a range, we choose to use the
 		   safer "deleteid" version */
-		return mpdclient_cmd_delete(c, start);
+		return RunDelete(start);
 
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
+	auto *c = GetConnection();
+	if (c == nullptr)
 		return false;
 
 	/* send the delete command to mpd; at the same time, get the
 	   new status (to verify the playlist id) */
 
-	if (!mpd_command_list_begin(connection, false) ||
-	    !mpd_send_delete_range(connection, start, end) ||
-	    !mpd_send_status(connection) ||
-	    !mpd_command_list_end(connection))
-		return c->HandleError();
+	if (!mpd_command_list_begin(c, false) ||
+	    !mpd_send_delete_range(c, start, end) ||
+	    !mpd_send_status(c) ||
+	    !mpd_command_list_end(c))
+		return HandleError();
 
-	c->events |= MPD_IDLE_QUEUE;
+	events |= MPD_IDLE_QUEUE;
 
-	struct mpd_status *status = mpdclient_recv_status(c);
-	if (status == nullptr)
+	const struct mpd_status *new_status = ReceiveStatus();
+	if (new_status == nullptr)
 		return false;
 
-	if (!mpd_response_finish(connection))
-		return c->HandleError();
+	if (!mpd_response_finish(c))
+		return HandleError();
 
-	if (mpd_status_get_queue_length(status) == c->playlist.size() - (end - start) &&
-	    mpd_status_get_queue_version(status) == c->playlist.version + 1) {
+	if (mpd_status_get_queue_length(new_status) == playlist.size() - (end - start) &&
+	    mpd_status_get_queue_version(new_status) == playlist.version + 1) {
 		/* the cheap route: match on the new playlist length
 		   and its version, we can keep our local playlist
 		   copy in sync */
-		c->playlist.version = mpd_status_get_queue_version(status);
+		playlist.version = mpd_status_get_queue_version(new_status);
 
 		/* remove the song from the local playlist */
 		while (end > start) {
 			--end;
 
 			/* remove references to the song */
-			if (c->song == &c->playlist[end])
-				c->song = nullptr;
+			if (current_song == &playlist[end])
+				current_song = nullptr;
 
-			c->playlist.RemoveIndex(end);
+			playlist.RemoveIndex(end);
 		}
 	}
 
@@ -767,42 +707,42 @@ mpdclient_cmd_delete_range(struct mpdclient *c, unsigned start, unsigned end)
 }
 
 bool
-mpdclient_cmd_move(struct mpdclient *c, unsigned dest_pos, unsigned src_pos)
+mpdclient::RunMove(unsigned dest_pos, unsigned src_pos) noexcept
 {
 	if (dest_pos == src_pos)
 		return true;
 
-	struct mpd_connection *connection = c->GetConnection();
-	if (connection == nullptr)
+	auto *c = GetConnection();
+	if (c == nullptr)
 		return false;
 
 	/* send the "move" command to MPD; at the same time, get the
 	   new status (to verify the playlist id) */
 
-	if (!mpd_command_list_begin(connection, false) ||
-	    !mpd_send_move(connection, src_pos, dest_pos) ||
-	    !mpd_send_status(connection) ||
-	    !mpd_command_list_end(connection))
-		return c->HandleError();
+	if (!mpd_command_list_begin(c, false) ||
+	    !mpd_send_move(c, src_pos, dest_pos) ||
+	    !mpd_send_status(c) ||
+	    !mpd_command_list_end(c))
+		return HandleError();
 
-	c->events |= MPD_IDLE_QUEUE;
+	events |= MPD_IDLE_QUEUE;
 
-	struct mpd_status *status = mpdclient_recv_status(c);
+	const struct mpd_status *new_status = ReceiveStatus();
 	if (status == nullptr)
 		return false;
 
-	if (!mpd_response_finish(connection))
-		return c->HandleError();
+	if (!mpd_response_finish(c))
+		return HandleError();
 
-	if (mpd_status_get_queue_length(status) == c->playlist.size() &&
-	    mpd_status_get_queue_version(status) == c->playlist.version + 1) {
+	if (mpd_status_get_queue_length(new_status) == playlist.size() &&
+	    mpd_status_get_queue_version(new_status) == playlist.version + 1) {
 		/* the cheap route: match on the new playlist length
 		   and its version, we can keep our local playlist
 		   copy in sync */
-		c->playlist.version = mpd_status_get_queue_version(status);
+		playlist.version = mpd_status_get_queue_version(new_status);
 
 		/* swap songs in the local playlist */
-		c->playlist.Move(dest_pos, src_pos);
+		playlist.Move(dest_pos, src_pos);
 	}
 
 	return true;
@@ -876,7 +816,7 @@ mpdclient::UpdateQueue()
 	}
 
 	playlist.version = mpd_status_get_queue_version(status);
-	song = nullptr;
+	current_song = nullptr;
 
 	return FinishCommand();
 }
@@ -895,7 +835,7 @@ mpdclient::UpdateQueueChanges()
 	while ((s = mpd_recv_song(c)) != nullptr) {
 		int pos = mpd_song_get_pos(s);
 
-		if (pos >= 0 && (guint)pos < playlist.size()) {
+		if (pos >= 0 && (unsigned)pos < playlist.size()) {
 			/* update song */
 			playlist.Replace(pos, *s);
 		} else {
@@ -914,7 +854,7 @@ mpdclient::UpdateQueueChanges()
 		playlist.RemoveIndex(playlist.size() - 1);
 	}
 
-	song = nullptr;
+	current_song = nullptr;
 	playlist.version = mpd_status_get_queue_version(status);
 
 	return FinishCommand();
