@@ -19,7 +19,6 @@
 
 #include "ncu.hxx"
 #include "config.h"
-#include "ncmpc_curses.h"
 
 #ifdef ENABLE_COLORS
 #include "Styles.hxx"
@@ -29,6 +28,8 @@
 #include "Options.hxx"
 #endif
 
+#include <curses.h>
+
 static SCREEN *ncu_screen;
 
 void
