@@ -20,9 +20,11 @@
 #include "plugin.hxx"
 #include "io/Path.hxx"
 #include "util/Compiler.h"
+#include "util/ScopeExit.hxx"
 #include "util/UriUtil.hxx"
 
-#include <glib.h>
+#include <boost/asio/steady_timer.hpp>
+#include <boost/asio/posix/stream_descriptor.hpp>
 
 #include <algorithm>
 #include <memory>
@@ -30,9 +32,10 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/stat.h>
-#include <sys/signal.h>
 #include <sys/wait.h>
 
 struct PluginCycle;
@@ -40,25 +43,37 @@ struct PluginCycle;
 struct PluginPipe {
 	PluginCycle *cycle;
 
-	/** the pipe to the plugin process, or -1 if none is currently
-	    open */
-	int fd = -1;
-	/** the GLib IO watch of #fd */
-	guint event_id;
+	/** the pipe to the plugin process */
+	boost::asio::posix::stream_descriptor fd;
+
 	/** the output of the current plugin */
 	std::string data;
 
-	~PluginPipe() {
+	std::array<char, 256> buffer;
+
+	PluginPipe(boost::asio::io_service &io_service) noexcept
+		:fd(io_service) {}
+
+	~PluginPipe() noexcept {
 		Close();
 	}
 
-	void Close() {
-		if (fd < 0)
+	void AsyncRead() noexcept {
+		fd.async_read_some(boost::asio::buffer(buffer),
+				   std::bind(&PluginPipe::OnRead, this,
+					     std::placeholders::_1,
+					     std::placeholders::_2));
+	}
+
+	void OnRead(const boost::system::error_code &error,
+		    std::size_t bytes_transferred) noexcept;
+
+	void Close() noexcept {
+		if (!fd.is_open())
 			return;
 
-		g_source_remove(event_id);
-		close(fd);
-		fd = -1;
+		fd.cancel();
+		fd.close();
 	}
 };
 
@@ -76,7 +91,7 @@ struct PluginCycle {
 
 	/** the index of the next plugin which is going to be
 	    invoked */
-	guint next_plugin = 0;
+	unsigned next_plugin = 0;
 
 	/** the pid of the plugin process, or -1 if none is currently
 	    running */
@@ -90,15 +105,37 @@ struct PluginCycle {
 	/** list of all error messages from failed plugins */
 	std::string all_errors;
 
-	PluginCycle(PluginList &_list, std::unique_ptr<char *[]> &&_argv,
-		    plugin_callback_t _callback, void *_callback_data)
+	boost::asio::steady_timer delayed_fail_timer;
+
+	PluginCycle(boost::asio::io_service &io_service,
+		    PluginList &_list, std::unique_ptr<char *[]> &&_argv,
+		    plugin_callback_t _callback, void *_callback_data) noexcept
 		:list(&_list), argv(std::move(_argv)),
 		 callback(_callback), callback_data(_callback_data),
-		 next_plugin(0) {}
+		 pipe_stdout(io_service), pipe_stderr(io_service),
+		 delayed_fail_timer(io_service) {}
+
+	void TryNextPlugin() noexcept;
+
+	void ScheduleDelayedFail() noexcept {
+		boost::system::error_code error;
+		delayed_fail_timer.expires_from_now(std::chrono::seconds(0),
+						    error);
+		delayed_fail_timer.async_wait(std::bind(&PluginCycle::OnDelayedFail,
+							this,
+							std::placeholders::_1));
+	}
+
+	void OnEof() noexcept;
+
+private:
+	int LaunchPlugin(const char *plugin_path) noexcept;
+
+	void OnDelayedFail(const boost::system::error_code &error) noexcept;
 };
 
 static bool
-register_plugin(PluginList *list, std::string &&path)
+register_plugin(PluginList *list, std::string &&path) noexcept
 {
 	struct stat st;
 	if (stat(path.c_str(), &st) < 0)
@@ -108,88 +145,84 @@ register_plugin(PluginList *list, std::string &&path)
 	return true;
 }
 
-bool
-plugin_list_load_directory(PluginList *list, const char *path)
+static constexpr bool
+ShallSkipDirectoryEntry(const char *name) noexcept
 {
-	GDir *dir = g_dir_open(path, 0, nullptr);
+	return name[0] == '.' && (name[1] == 0 || (name[1] == '.' && name[2] == 0));
+}
+
+bool
+plugin_list_load_directory(PluginList *list, const char *path) noexcept
+{
+	DIR *dir = opendir(path);
 	if (dir == nullptr)
 		return false;
 
-	const char *name;
-	while ((name = g_dir_read_name(dir)) != nullptr) {
-		register_plugin(list, BuildPath(path, name));
-	}
+	AtScopeExit(dir) { closedir(dir); };
 
-	g_dir_close(dir);
+	while (const auto *e = readdir(dir)) {
+		const char *name = e->d_name;
+		if (!ShallSkipDirectoryEntry(name))
+			register_plugin(list, BuildPath(path, name));
+	}
 
 	std::sort(list->plugins.begin(), list->plugins.end());
 
 	return true;
 }
 
-static void
-next_plugin(PluginCycle *cycle);
-
-static void
-plugin_eof(PluginCycle *cycle, PluginPipe *p)
+void
+PluginCycle::OnEof() noexcept
 {
-	close(p->fd);
-	p->fd = -1;
-
 	/* Only if both pipes are have EOF status we are done */
-	if (cycle->pipe_stdout.fd != -1 || cycle->pipe_stderr.fd != -1)
+	if (pipe_stdout.fd.is_open() || pipe_stderr.fd.is_open())
 		return;
 
-	int status, ret = waitpid(cycle->pid, &status, 0);
-	cycle->pid = -1;
+	int status, ret = waitpid(pid, &status, 0);
+	pid = -1;
 
 	if (ret < 0 || !WIFEXITED(status) || WEXITSTATUS(status) != 0) {
 		/* If we encountered an error other than service unavailable
 		 * (69), log it for later. If all plugins fail, we may get
 		 * some hints for debugging.*/
-		if (!cycle->pipe_stderr.data.empty() &&
+		if (!pipe_stderr.data.empty() &&
 		    WEXITSTATUS(status) != 69) {
-			cycle->all_errors += "*** ";
-			cycle->all_errors += cycle->argv[0];
-			cycle->all_errors += " ***\n\n";
-			cycle->all_errors += cycle->pipe_stderr.data;
-			cycle->all_errors += "\n";
+			all_errors += "*** ";
+			all_errors += argv[0];
+			all_errors += " ***\n\n";
+			all_errors += pipe_stderr.data;
+			all_errors += "\n";
 		}
 
 		/* the plugin has failed */
-		cycle->pipe_stdout.data.clear();
-		cycle->pipe_stderr.data.clear();
+		pipe_stdout.data.clear();
+		pipe_stderr.data.clear();
 
-		next_plugin(cycle);
+		TryNextPlugin();
 	} else {
 		/* success: invoke the callback */
-		cycle->callback(std::move(cycle->pipe_stdout.data), true,
-				cycle->argv[0], cycle->callback_data);
+		callback(std::move(pipe_stdout.data), true,
+			 argv[0], callback_data);
 	}
 }
 
-static gboolean
-plugin_data(gcc_unused GIOChannel *source,
-	    gcc_unused GIOCondition condition, gpointer data)
+void
+PluginPipe::OnRead(const boost::system::error_code &error,
+		   std::size_t bytes_transferred) noexcept
 {
-	auto *p = (PluginPipe *)data;
-	assert(p->fd >= 0);
+	if (error) {
+		if (error == boost::asio::error::operation_aborted)
+			/* this object has already been deleted; bail out
+			   quickly without touching anything */
+			return;
 
-	PluginCycle *cycle = p->cycle;
-	assert(cycle != nullptr);
-	assert(cycle->pid > 0);
-
-	char buffer[256];
-	ssize_t nbytes = condition & G_IO_IN
-		? read(p->fd, buffer, sizeof(buffer))
-		: 0;
-	if (nbytes <= 0) {
-		plugin_eof(cycle, p);
-		return false;
+		fd.close();
+		cycle->OnEof();
+		return;
 	}
 
-	p->data.append(buffer, nbytes);
-	return true;
+	data.append(&buffer.front(), bytes_transferred);
+	AsyncRead();
 }
 
 /**
@@ -199,46 +232,40 @@ plugin_data(gcc_unused GIOChannel *source,
  * Instead, install a timer which calls the plugin callback in the
  * moment after.
  */
-static gboolean
-plugin_delayed_fail(gpointer data)
+void
+PluginCycle::OnDelayedFail(const boost::system::error_code &error) noexcept
 {
-	auto *cycle = (PluginCycle *)data;
+	if (error)
+		return;
 
-	assert(cycle != nullptr);
-	assert(cycle->pipe_stdout.fd < 0);
-	assert(cycle->pipe_stderr.fd < 0);
-	assert(cycle->pid < 0);
+	assert(!pipe_stdout.fd.is_open());
+	assert(!pipe_stderr.fd.is_open());
+	assert(pid < 0);
 
-	cycle->callback(std::move(cycle->all_errors), false, nullptr,
-			cycle->callback_data);
-
-	return false;
+	callback(std::move(all_errors), false, nullptr,
+		 callback_data);
 }
 
 static void
-plugin_fd_add(PluginCycle *cycle, PluginPipe *p, int fd)
+plugin_fd_add(PluginCycle *cycle, PluginPipe *p, int fd) noexcept
 {
 	p->cycle = cycle;
-	p->fd = fd;
-	GIOChannel *channel = g_io_channel_unix_new(fd);
-	p->event_id = g_io_add_watch(channel, GIOCondition(G_IO_IN|G_IO_HUP),
-				     plugin_data, p);
-	g_io_channel_unref(channel);
+	p->fd.assign(fd);
+	p->AsyncRead();
 }
 
-static int
-start_plugin(PluginCycle *cycle, const char *plugin_path)
+int
+PluginCycle::LaunchPlugin(const char *plugin_path) noexcept
 {
-	assert(cycle != nullptr);
-	assert(cycle->pid < 0);
-	assert(cycle->pipe_stdout.fd < 0);
-	assert(cycle->pipe_stderr.fd < 0);
-	assert(cycle->pipe_stdout.data.empty());
-	assert(cycle->pipe_stderr.data.empty());
+	assert(pid < 0);
+	assert(!pipe_stdout.fd.is_open());
+	assert(!pipe_stderr.fd.is_open());
+	assert(pipe_stdout.data.empty());
+	assert(pipe_stderr.data.empty());
 
 	/* set new program name, but free the one from the previous
 	   plugin */
-	cycle->argv[0] = const_cast<char *>(GetUriFilename(plugin_path));
+	argv[0] = const_cast<char *>(GetUriFilename(plugin_path));
 
 	int fds_stdout[2];
 	if (pipe(fds_stdout) < 0)
@@ -251,7 +278,7 @@ start_plugin(PluginCycle *cycle, const char *plugin_path)
 		return -1;
 	}
 
-	pid_t pid = fork();
+	pid = fork();
 
 	if (pid < 0) {
 		close(fds_stdout[0]);
@@ -271,49 +298,47 @@ start_plugin(PluginCycle *cycle, const char *plugin_path)
 		close(0);
 		/* XXX close other fds? */
 
-		execv(plugin_path, cycle->argv.get());
+		execv(plugin_path, argv.get());
 		_exit(1);
 	}
 
 	close(fds_stdout[1]);
 	close(fds_stderr[1]);
 
-	cycle->pid = pid;
-
 	/* XXX CLOEXEC? */
 
-	plugin_fd_add(cycle, &cycle->pipe_stdout, fds_stdout[0]);
-	plugin_fd_add(cycle, &cycle->pipe_stderr, fds_stderr[0]);
+	plugin_fd_add(this, &pipe_stdout, fds_stdout[0]);
+	plugin_fd_add(this, &pipe_stderr, fds_stderr[0]);
 
 	return 0;
 }
 
-static void
-next_plugin(PluginCycle *cycle)
+void
+PluginCycle::TryNextPlugin() noexcept
 {
-	assert(cycle->pid < 0);
-	assert(cycle->pipe_stdout.fd < 0);
-	assert(cycle->pipe_stderr.fd < 0);
-	assert(cycle->pipe_stdout.data.empty());
-	assert(cycle->pipe_stderr.data.empty());
+	assert(pid < 0);
+	assert(!pipe_stdout.fd.is_open());
+	assert(!pipe_stderr.fd.is_open());
+	assert(pipe_stdout.data.empty());
+	assert(pipe_stderr.data.empty());
 
-	if (cycle->next_plugin >= cycle->list->plugins.size()) {
+	if (next_plugin >= list->plugins.size()) {
 		/* no plugins left */
-		g_idle_add(plugin_delayed_fail, cycle);
+		ScheduleDelayedFail();
 		return;
 	}
 
 	const char *plugin_path = (const char *)
-		cycle->list->plugins[cycle->next_plugin++].c_str();
-	if (start_plugin(cycle, plugin_path) < 0) {
+		list->plugins[next_plugin++].c_str();
+	if (LaunchPlugin(plugin_path) < 0) {
 		/* system error */
-		g_idle_add(plugin_delayed_fail, cycle);
+		ScheduleDelayedFail();
 		return;
 	}
 }
 
 static auto
-make_argv(const char*const* args)
+make_argv(const char*const* args) noexcept
 {
 	unsigned num = 0;
 	while (args[num] != nullptr)
@@ -337,20 +362,21 @@ make_argv(const char*const* args)
 }
 
 PluginCycle *
-plugin_run(PluginList *list, const char *const*args,
-	   plugin_callback_t callback, void *callback_data)
+plugin_run(boost::asio::io_service &io_service,
+	   PluginList *list, const char *const*args,
+	   plugin_callback_t callback, void *callback_data) noexcept
 {
 	assert(args != nullptr);
 
-	auto *cycle = new PluginCycle(*list, make_argv(args),
+	auto *cycle = new PluginCycle(io_service, *list, make_argv(args),
 				      callback, callback_data);
-	next_plugin(cycle);
+	cycle->TryNextPlugin();
 
 	return cycle;
 }
 
 void
-plugin_stop(PluginCycle *cycle)
+plugin_stop(PluginCycle *cycle) noexcept
 {
 	if (cycle->pid > 0) {
 		/* kill the plugin process */
