@@ -68,7 +68,8 @@ PlaylistNameCompletion::Post(gcc_unused const char *value, Range range)
 #endif
 
 int
-playlist_save(struct mpdclient *c, char *name, char *defaultname)
+playlist_save(struct mpdclient *c, const char *name,
+	      const char *defaultname) noexcept
 {
 	std::string filename;
 
