@@ -25,8 +25,9 @@
 #include "keyboard.hxx"
 #include "i18n.h"
 #include "Options.hxx"
+#include "util/LocaleString.hxx"
 
-#include <glib.h>
+#include <ctype.h>
 
 #define FIND_PROMPT  _("Find")
 #define RFIND_PROMPT _("Find backward")
@@ -34,8 +35,8 @@
 
 /* query user for a string and find it in a list window */
 bool
-screen_find(ScreenManager &screen, ListWindow *lw, Command findcmd,
-	    const ListText &text)
+screen_find(ScreenManager &screen, ListWindow &lw, Command findcmd,
+	    const ListText &text) noexcept
 {
 	bool found;
 	const char *prompt = FIND_PROMPT;
@@ -67,14 +68,14 @@ screen_find(ScreenManager &screen, ListWindow *lw, Command findcmd,
 			return true;
 
 		found = reversed
-			? lw->ReverseFind(text,
-					  screen.findbuf.c_str(),
-					  options.find_wrap,
-					  options.bell_on_wrap)
-			: lw->Find(text,
-				   screen.findbuf.c_str(),
-				   options.find_wrap,
-				   options.bell_on_wrap);
+			? lw.ReverseFind(text,
+					 screen.findbuf.c_str(),
+					 options.find_wrap,
+					 options.bell_on_wrap)
+			: lw.Find(text,
+				  screen.findbuf.c_str(),
+				  options.find_wrap,
+				  options.bell_on_wrap);
 		if (!found) {
 			screen_status_printf(_("Unable to find \'%s\'"),
 					     screen.findbuf.c_str());
@@ -90,15 +91,14 @@ screen_find(ScreenManager &screen, ListWindow *lw, Command findcmd,
 /* query user for a string and jump to the entry
  * which begins with this string while the users types */
 void
-screen_jump(ScreenManager &screen, ListWindow *lw,
+screen_jump(ScreenManager &screen, ListWindow &lw,
 	    const ListText &text,
-	    const ListRenderer &renderer)
+	    const ListRenderer &renderer) noexcept
 {
 	constexpr size_t WRLN_MAX_LINE_SIZE = 1024;
 	int key = 65;
 
 	char buffer[WRLN_MAX_LINE_SIZE];
-	std::fill_n(buffer, WRLN_MAX_LINE_SIZE, 0);
 
 	/* In screen.findbuf is the whole string which is displayed in the status_window
 	 * and search_str is the string the user entered (without the prompt) */
@@ -109,31 +109,29 @@ screen_jump(ScreenManager &screen, ListWindow *lw,
 		key = screen_getch(buffer);
 		/* if backspace or delete was pressed, process instead of ending loop */
 		if (key == KEY_BACKSPACE || key == KEY_DC) {
-			int i;
-			if (search_str <= g_utf8_find_prev_char(buffer, iter))
-				iter = g_utf8_find_prev_char(buffer, iter);
-			for (i = 0; *(iter + i) != '\0'; i++)
-				*(iter + i) = '\0';
+			const char *prev = PrevCharMB(buffer, iter);
+			if (search_str <= prev)
+				iter = const_cast<char *>(prev);
+			*iter = '\0';
 			continue;
 		}
 		/* if a control key was pressed, end loop */
-		else if (g_ascii_iscntrl(key) || key == KEY_NPAGE || key == KEY_PPAGE) {
+		else if (iscntrl(key) || key == KEY_NPAGE || key == KEY_PPAGE) {
 			break;
 		}
-		else {
-			*iter = key;
-			if (iter < buffer + WRLN_MAX_LINE_SIZE - 3)
-				++iter;
+		else if (iter < buffer + WRLN_MAX_LINE_SIZE - 3) {
+			*iter++ = key;
+			*iter = '\0';
 		}
-		lw->Jump(text, search_str);
+		lw.Jump(text, search_str);
 
 		/* repaint the list_window */
-		lw->Paint(renderer);
-		wrefresh(lw->w);
+		lw.Paint(renderer);
+		wrefresh(lw.w);
 	}
 
 	screen.findbuf = search_str;
 
 	/* ncmpc should get the command */
-	keyboard_unread(key);
+	keyboard_unread(screen.get_io_service(), key);
 }
