@@ -25,7 +25,6 @@
 #include "i18n.h"
 #include "Options.hxx"
 #include "Styles.hxx"
-#include "Event.hxx"
 
 #include <stdlib.h>
 
@@ -33,27 +32,37 @@
 static const unsigned SCREEN_MIN_COLS = 14;
 static const unsigned SCREEN_MIN_ROWS = 5;
 
-ScreenManager::ScreenManager()
-	:layout({std::max<unsigned>(COLS, SCREEN_MIN_COLS),
+ScreenManager::ScreenManager(boost::asio::io_service &_io_service)
+	:io_service(_io_service),
+	 layout({std::max<unsigned>(COLS, SCREEN_MIN_COLS),
 		 std::max<unsigned>(LINES, SCREEN_MIN_ROWS)}),
 	 title_bar({layout.title_x, layout.title_y}, layout.size.width),
 	 main_window({layout.main_x, layout.main_y}, layout.GetMainSize()),
 	 progress_bar({layout.progress_x, layout.GetProgressY()}, layout.size.width),
-	 status_bar({layout.status_x, layout.GetStatusY()}, layout.size.width),
+	 status_bar(io_service,
+		    {layout.status_x, layout.GetStatusY()}, layout.size.width),
 	 mode_fn_prev(&screen_queue)
 {
 	buf_size = layout.size.width;
-	buf = (char *)g_malloc(buf_size);
+	buf = new char[buf_size];
 
 	if (!options.hardware_cursor)
 		leaveok(main_window.w, true);
 
 	keypad(main_window.w, true);
+
+#ifdef ENABLE_COLORS
+	if (options.enable_colors) {
+		/* set background attributes */
+		wbkgd(stdscr, COLOR_PAIR(Style::LIST));
+		wbkgd(main_window.w, COLOR_PAIR(Style::LIST));
+	}
+#endif
 }
 
 ScreenManager::~ScreenManager()
 {
-	g_free(buf);
+	delete[] buf;
 }
 
 void
@@ -90,8 +99,8 @@ ScreenManager::OnResize()
 			    layout.size.width);
 
 	buf_size = layout.size.width;
-	g_free(buf);
-	buf = (char *)g_malloc(buf_size);
+	delete[] buf;
+	buf = new char[buf_size];
 
 	/* resize all screens */
 	current_page->second->Resize(main_window.size);
@@ -106,14 +115,6 @@ ScreenManager::OnResize()
 void
 ScreenManager::Init(struct mpdclient *c)
 {
-#ifdef ENABLE_COLORS
-	if (options.enable_colors) {
-		/* set background attributes */
-		wbkgd(stdscr, COLOR_PAIR(Style::LIST));
-		wbkgd(main_window.w, COLOR_PAIR(Style::LIST));
-	}
-#endif
-
 	current_page = MakePage(screen_queue);
 	current_page->second->OnOpen(*c);
 }
