@@ -43,6 +43,7 @@
 
 class CommandKeysPage final : public ListPage, ListText {
 	ScreenManager &screen;
+	Page *const parent;
 
 	const KeyBindings *bindings;
 	KeyBinding *binding;
@@ -57,8 +58,9 @@ class CommandKeysPage final : public ListPage, ListText {
 	unsigned subcmd_n_keys = 0;
 
 public:
-	CommandKeysPage(ScreenManager &_screen, WINDOW *w, Size size)
-		:ListPage(w, size), screen(_screen) {}
+	CommandKeysPage(ScreenManager &_screen, Page *_parent,
+			WINDOW *w, Size size) noexcept
+		:ListPage(w, size), screen(_screen), parent(_parent) {}
 
 	void SetCommand(KeyBindings *_bindings, unsigned _cmd) {
 		bindings = _bindings;
@@ -122,15 +124,15 @@ private:
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-	void Paint() const override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 
 private:
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const override;
+				    unsigned i) const noexcept override;
 };
 
 /* TODO: rename to check_n_keys / subcmd_count_keys? */
@@ -220,7 +222,7 @@ CommandKeysPage::AddKey()
 
 const char *
 CommandKeysPage::GetListItemText(char *buffer, size_t size,
-				 unsigned idx) const
+				 unsigned idx) const noexcept
 {
 	if (idx == subcmd_item_up())
 		return "[..]";
@@ -240,13 +242,13 @@ CommandKeysPage::GetListItemText(char *buffer, size_t size,
 }
 
 void
-CommandKeysPage::OnOpen(gcc_unused struct mpdclient &c)
+CommandKeysPage::OnOpen(gcc_unused struct mpdclient &c) noexcept
 {
 	// TODO
 }
 
 const char *
-CommandKeysPage::GetTitle(char *str, size_t size) const
+CommandKeysPage::GetTitle(char *str, size_t size) const noexcept
 {
 	snprintf(str, size, _("Edit keys for %s"),
 		 get_key_command_name(Command(subcmd)));
@@ -254,7 +256,7 @@ CommandKeysPage::GetTitle(char *str, size_t size) const
 }
 
 void
-CommandKeysPage::Paint() const
+CommandKeysPage::Paint() const noexcept
 {
 	lw.Paint(TextListRenderer(*this));
 }
@@ -271,7 +273,8 @@ CommandKeysPage::OnCommand(struct mpdclient &c, Command cmd)
 	switch(cmd) {
 	case Command::PLAY:
 		if (lw.selected == subcmd_item_up()) {
-			screen.OnCommand(c, Command::GO_PARENT_DIRECTORY);
+			if (parent != nullptr)
+				return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
 		} else if (lw.selected == subcmd_item_add()) {
 			AddKey();
 		} else {
@@ -292,7 +295,7 @@ CommandKeysPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SetDirty();
 		return true;
 
@@ -364,15 +367,15 @@ public:
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-	void Paint() const override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void Paint() const noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
-	const char *GetTitle(char *s, size_t size) const override;
+	const char *GetTitle(char *s, size_t size) const noexcept override;
 
 private:
 	/* virtual methods from class ListText */
 	const char *GetListItemText(char *buffer, size_t size,
-				    unsigned i) const override;
+				    unsigned i) const noexcept override;
 };
 
 bool
@@ -427,7 +430,8 @@ CommandListPage::Save()
 }
 
 const char *
-CommandListPage::GetListItemText(char *buffer, size_t size, unsigned idx) const
+CommandListPage::GetListItemText(char *buffer, size_t size,
+				 unsigned idx) const noexcept
 {
 	if (idx == command_item_apply())
 		return _("===> Apply key bindings ");
@@ -458,7 +462,7 @@ CommandListPage::GetListItemText(char *buffer, size_t size, unsigned idx) const
 }
 
 void
-CommandListPage::OnOpen(gcc_unused struct mpdclient &c)
+CommandListPage::OnOpen(gcc_unused struct mpdclient &c) noexcept
 {
 	if (bindings == nullptr)
 		bindings = new KeyBindings(GetGlobalKeyBindings());
@@ -467,13 +471,13 @@ CommandListPage::OnOpen(gcc_unused struct mpdclient &c)
 }
 
 const char *
-CommandListPage::GetTitle(char *, size_t) const
+CommandListPage::GetTitle(char *, size_t) const noexcept
 {
 	return _("Edit key bindings");
 }
 
 void
-CommandListPage::Paint() const
+CommandListPage::Paint() const noexcept
 {
 	lw.Paint(TextListRenderer(*this));
 }
@@ -504,7 +508,7 @@ CommandListPage::OnCommand(struct mpdclient &c, Command cmd)
 	case Command::LIST_RFIND:
 	case Command::LIST_FIND_NEXT:
 	case Command::LIST_RFIND_NEXT:
-		screen_find(screen, &lw, cmd, *this);
+		screen_find(screen, lw, cmd, *this);
 		SetDirty();
 		return true;
 
@@ -523,12 +527,12 @@ public:
 	KeyDefPage(ScreenManager &screen, WINDOW *_w, Size size)
 		:ProxyPage(_w),
 		 command_list_page(screen, _w, size),
-		 command_keys_page(screen, _w, size) {}
+		 command_keys_page(screen, this, _w, size) {}
 
 public:
 	/* virtual methods from class Page */
-	void OnOpen(struct mpdclient &c) override;
-	void OnClose() override;
+	void OnOpen(struct mpdclient &c) noexcept override;
+	void OnClose() noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
 };
 
@@ -539,7 +543,7 @@ keydef_init(ScreenManager &screen, WINDOW *w, Size size)
 }
 
 void
-KeyDefPage::OnOpen(struct mpdclient &c)
+KeyDefPage::OnOpen(struct mpdclient &c) noexcept
 {
 	ProxyPage::OnOpen(c);
 
@@ -548,7 +552,7 @@ KeyDefPage::OnOpen(struct mpdclient &c)
 }
 
 void
-KeyDefPage::OnClose()
+KeyDefPage::OnClose() noexcept
 {
 	if (command_list_page.IsModified())
 		screen_status_message(_("Note: Did you forget to \'Apply\' your changes?"));
