@@ -23,7 +23,7 @@
 #include "time_format.hxx"
 #include "hscroll.hxx"
 #include "config.h"
-#include "util/StringUTF8.hxx"
+#include "util/LocaleString.hxx"
 
 #include <mpd/client.h>
 
@@ -34,7 +34,7 @@ paint_song_row(WINDOW *w, gcc_unused unsigned y, unsigned width,
 	       bool selected, bool highlight, const struct mpd_song *song,
 	       gcc_unused class hscroll *hscroll, const char *format)
 {
-	char buffer[width * 4];
+	char buffer[1024];
 
 	strfsong(buffer, sizeof(buffer), format, song);
 	row_paint_text(w, width, highlight ? Style::LIST_BOLD : Style::LIST,
@@ -51,7 +51,7 @@ paint_song_row(WINDOW *w, gcc_unused unsigned y, unsigned width,
 		waddstr(w, duration);
 	}
 
-	if (hscroll != nullptr && utf8_width(buffer) >= width) {
+	if (hscroll != nullptr && StringWidthMB(buffer) >= width) {
 		hscroll->Set(0, y, width, buffer,
 			     highlight ? Style::LIST_BOLD : Style::LIST,
 			     selected ? A_REVERSE : 0);
